@@ -70,6 +70,7 @@
 
 #include "igt_core.h"
 #include "igt_aux.h"
+#include "igt_hook.h"
 #include "igt_sysfs.h"
 #include "igt_sysrq.h"
 #include "igt_rc.h"
@@ -77,6 +78,7 @@
 #include "igt_map.h"
 #include "igt_device_scan.h"
 #include "igt_thread.h"
+#include "igt_vec.h"
 #include "runnercomms.h"
 
 #define UNW_LOCAL_ONLY
@@ -217,11 +219,10 @@
  * On top of that tests may report unexpected results and minor issues to
  * stderr. If stderr is non-empty the test result should be treated as "warn".
  *
- * The test lists are generated at build time. Simple testcases are listed in
- * tests/single-tests.txt and tests with subtests are listed in
- * tests/multi-tests.txt. When running tests with subtest from a test runner it
- * is recommend to run each subtest individually, since otherwise the return
- * code will only reflect the overall result.
+ * The list of test executables is generated at build time and stored in
+ * tests/test-list.txt. When running tests with subtest from a test runner it is
+ * recommend to run each subtest individually, since otherwise the return code
+ * will only reflect the overall result.
  *
  * To do that obtain the lists of subtests with "--list-subtests", which can be
  * run as non-root and doesn't require a DRM driver to be loaded (or any GPU to
@@ -242,6 +243,9 @@
  * - '*,!basic*' match any subtest not starting basic
  * - 'basic*,!basic-render*' match any subtest starting basic but not starting basic-render
  *
+ * It is possible to run a shell script at certain points of test execution with
+ * "--hook". See the usage description with "--help-hook" for details.
+ *
  * # Configuration
  *
  * Some of IGT's behavior can be configured through a configuration file.
@@ -274,6 +278,8 @@ static unsigned int exit_handler_count;
 const char *igt_interactive_debug;
 bool igt_skip_crc_compare;
 
+static struct igt_hook *igt_hook = NULL;
+
 /* subtests helpers */
 static bool show_testlist = false;
 static bool list_subtests = false;
@@ -339,6 +345,8 @@ enum {
 	OPT_INTERACTIVE_DEBUG,
 	OPT_SKIP_CRC,
 	OPT_TRACE_OOPS,
+	OPT_HOOK,
+	OPT_HELP_HOOK,
 	OPT_DEVICE,
 	OPT_VERSION,
 	OPT_HELP = 'h'
@@ -811,6 +819,8 @@ static void common_exit_handler(int sig)
 		bind_fbcon(true);
 	}
 
+	igt_hook_free(igt_hook);
+
 	/* When not killed by a signal check that igt_exit() has been properly
 	 * called. */
 	assert(sig != 0 || igt_exit_called || igt_is_aborting);
@@ -908,6 +918,8 @@ static void print_usage(const char *help_str, bool output_on_stderr)
 		   "  --interactive-debug[=domain]\n"
 		   "  --skip-crc-compare\n"
 		   "  --trace-on-oops\n"
+		   "  --hook [<events>:]<cmd>\n"
+		   "  --help-hook\n"
 		   "  --help-description\n"
 		   "  --describe\n"
 		   "  --device filters\n"
@@ -1091,6 +1103,8 @@ static int common_init(int *argc, char **argv,
 		{"interactive-debug", optional_argument, NULL, OPT_INTERACTIVE_DEBUG},
 		{"skip-crc-compare",  no_argument,       NULL, OPT_SKIP_CRC},
 		{"trace-on-oops",     no_argument,       NULL, OPT_TRACE_OOPS},
+		{"hook",              required_argument, NULL, OPT_HOOK},
+		{"help-hook",         no_argument,       NULL, OPT_HELP_HOOK},
 		{"device",            required_argument, NULL, OPT_DEVICE},
 		{"version",           no_argument,       NULL, OPT_VERSION},
 		{"help",              no_argument,       NULL, OPT_HELP},
@@ -1102,10 +1116,12 @@ static int common_init(int *argc, char **argv,
 	struct option *combined_opts;
 	int extra_opt_count;
 	int all_opt_count;
+	struct igt_vec hook_strs;
 	int ret = 0;
 
 	common_init_env();
 	IGT_INIT_LIST_HEAD(&subgroup_descriptions);
+	igt_vec_init(&hook_strs, sizeof(char *));
 
 	command_str = argv[0];
 	if (strrchr(command_str, '/'))
@@ -1226,6 +1242,14 @@ static int common_init(int *argc, char **argv,
 		case OPT_TRACE_OOPS:
 			show_ftrace = true;
 			break;
+		case OPT_HOOK:
+			assert(optarg);
+			igt_vec_push(&hook_strs, &optarg);
+			break;
+		case OPT_HELP_HOOK:
+			igt_hook_print_help(stdout, "--hook");
+			ret = -1;
+			goto out;
 		case OPT_DEVICE:
 			assert(optarg);
 			/* if set by env IGT_DEVICE we need to free it */
@@ -1254,11 +1278,23 @@ static int common_init(int *argc, char **argv,
 		}
 	}
 
+	if (igt_vec_length(&hook_strs)) {
+		ret = igt_hook_create(hook_strs.elems, igt_vec_length(&hook_strs), &igt_hook);
+
+		if (ret) {
+			igt_critical("Failed to initialize hook data: %s\n",
+				     igt_hook_error_str(ret));
+			ret = -2;
+			goto out;
+		}
+	}
+
 	common_init_config();
 
 out:
 	free(short_opts);
 	free(combined_opts);
+	igt_vec_fini(&hook_strs);
 
 	/* exit immediately if this test has no subtests and a subtest or the
 	 * list of subtests has been requested */
@@ -1275,14 +1311,25 @@ out:
 			exit(IGT_EXIT_INVALID);
 	}
 
-	if (ret < 0)
+	if (ret < 0) {
+		if (igt_hook) {
+			igt_hook_free(igt_hook);
+			igt_hook = NULL;
+		}
+
 		/* exit with no error for -h/--help */
 		exit(ret == -1 ? 0 : IGT_EXIT_INVALID);
+	}
 
 	if (!igt_only_list_subtests()) {
 		bind_fbcon(false);
 		igt_kmsg(KMSG_INFO "%s: executing\n", command_str);
 		print_version();
+		igt_srandom();
+
+		igt_hook_event_notify(igt_hook, &(struct igt_hook_evt){
+			.evt_type = IGT_HOOK_PRE_TEST,
+			.target_name = command_str });
 
 		sync();
 		oom_adjust_for_doom();
@@ -1487,6 +1534,11 @@ bool __igt_run_subtest(const char *subtest_name, const char *file, const int lin
 	igt_thread_clear_fail_state();
 
 	igt_gettime(&subtest_time);
+
+	igt_hook_event_notify(igt_hook, &(struct igt_hook_evt){
+		.evt_type = IGT_HOOK_PRE_SUBTEST,
+		.target_name = subtest_name });
+
 	return (in_subtest = subtest_name);
 }
 
@@ -1517,6 +1569,11 @@ bool __igt_run_dynamic_subtest(const char *dynamic_subtest_name)
 	_igt_dynamic_tests_executed++;
 
 	igt_gettime(&dynamic_subtest_time);
+
+	igt_hook_event_notify(igt_hook, &(struct igt_hook_evt){
+		.evt_type = IGT_HOOK_PRE_DYN_SUBTEST,
+		.target_name = dynamic_subtest_name });
+
 	return (in_dynamic_subtest = dynamic_subtest_name);
 }
 
@@ -1602,6 +1659,12 @@ __noreturn static void exit_subtest(const char *result)
 	struct timespec *thentime = in_dynamic_subtest ? &dynamic_subtest_time : &subtest_time;
 	jmp_buf *jmptarget = in_dynamic_subtest ? &igt_dynamic_jmpbuf : &igt_subtest_jmpbuf;
 
+	igt_hook_event_notify(igt_hook, &(struct igt_hook_evt){
+		.evt_type = (in_dynamic_subtest
+				? IGT_HOOK_POST_DYN_SUBTEST
+				: IGT_HOOK_POST_SUBTEST),
+		.result = result });
+
 	if (!igt_thread_is_main()) {
 		igt_thread_fail();
 		pthread_exit(NULL);
@@ -2274,6 +2337,7 @@ void __igt_abort(const char *domain, const char *file, const int line,
 void igt_exit(void)
 {
 	int tmp;
+	const char *result;
 
 	if (!test_with_subtests)
 		igt_thread_assert_no_failures();
@@ -2318,12 +2382,7 @@ void igt_exit(void)
 
 	assert(waitpid(-1, &tmp, WNOHANG) == -1 && errno == ECHILD);
 
-	if (!test_with_subtests) {
-		struct timespec now;
-		const char *result;
-
-		igt_gettime(&now);
-
+	if (!test_with_subtests || igt_hook) {
 		switch (igt_exitcode) {
 			case IGT_EXIT_SUCCESS:
 				result = "SUCCESS";
@@ -2334,6 +2393,12 @@ void igt_exit(void)
 			default:
 				result = "FAIL";
 		}
+	}
+
+	if (!test_with_subtests) {
+		struct timespec now;
+
+		igt_gettime(&now);
 
 		if (test_multi_fork_child) /* parent will do the yelling */
 			_log_line_fprintf(stdout, "dyn_child pid:%d (%.3fs) ends with err=%d\n",
@@ -2344,6 +2409,10 @@ void igt_exit(void)
 					  result, igt_time_elapsed(&subtest_time, &now));
 	}
 
+	igt_hook_event_notify(igt_hook, &(struct igt_hook_evt){
+		.evt_type = IGT_HOOK_POST_TEST,
+		.result = result });
+
 	exit(igt_exitcode);
 }
 
@@ -3414,21 +3483,73 @@ void igt_srandom(void)
 }
 
 /* IGT wrappers around libpciaccess init/cleanup functions */
+static bool pci_system_initialized;
+static pthread_mutex_t pci_system_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+void igt_pci_system_cleanup(void)
+{
+	pthread_mutex_lock(&pci_system_mutex);
+	if (pci_system_initialized) {
+		pci_system_cleanup();
+		pci_system_initialized = false;
+	}
+	pthread_mutex_unlock(&pci_system_mutex);
+}
 
 static void pci_system_exit_handler(int sig)
 {
-	pci_system_cleanup();
-}
-
-static void __pci_system_init(void)
-{
-	if (!igt_warn_on_f(pci_system_init(), "Could not initialize libpciaccess global data\n"))
-		igt_install_exit_handler(pci_system_exit_handler);
+	igt_pci_system_cleanup();
 }
 
 int igt_pci_system_init(void)
 {
-	static pthread_once_t once_control = PTHREAD_ONCE_INIT;
+	int ret = 0;
+	bool install_handler = false;
 
-	return pthread_once(&once_control, __pci_system_init);
+	pthread_mutex_lock(&pci_system_mutex);
+	if (!pci_system_initialized) {
+		ret = igt_warn_on_f(pci_system_init(),
+				    "Could not initialize libpciaccess global data\n");
+		if (!ret) {
+			pci_system_initialized = true;
+			install_handler = true;
+		}
+	}
+	pthread_mutex_unlock(&pci_system_mutex);
+
+	if (install_handler)
+		igt_install_exit_handler(pci_system_exit_handler);
+
+	return ret;
+}
+
+int igt_pci_system_reinit(void)
+{
+	igt_pci_system_cleanup();
+
+	return igt_pci_system_init();
+}
+
+/**
+ * igt_emit_ignore_dmesg_regex:
+ * @ignore_dmesg: string regex
+ *
+ * Emits a string for igt_runner to ignore next dmesg warns or errors which
+ * matches it.
+ */
+void igt_emit_ignore_dmesg_regex(const char *ignore_dmesg_regex)
+{
+	static const char mark_ignore_dmesg[] = "add ignored dmesg regex: ";
+	GError *err = NULL;
+	GRegex *re;
+
+	re = g_regex_new(ignore_dmesg_regex, G_REGEX_OPTIMIZE, 0, &err);
+	if (err) {
+		igt_debug("regexp: '%s'\n", err->message);
+		g_error_free(err);
+		igt_assert_f(re, "Error in regexp\n");
+	}
+
+	g_regex_unref(re);
+	igt_kmsg(KMSG_INFO "%s%s\n", mark_ignore_dmesg, ignore_dmesg_regex);
 }
