@@ -208,19 +208,53 @@ char *igt_debugfs_path(int device, char *path, int pathlen)
  */
 int igt_debugfs_dir(int device)
 {
+	int debugfs_dir_fd;
 	char path[200];
 
+	if (igt_debug_on(!igt_debugfs_path(device, path, sizeof(path))))
+		return -1;
+
+	debugfs_dir_fd = open(path, O_RDONLY);
+	igt_debug_on_f(debugfs_dir_fd < 0, "path: %s\n", path);
+
+	return debugfs_dir_fd;
+}
+
+/**
+ * igt_debugfs_gt_dir:
+ * @device: fd of the device
+ * @gt: GT instance number
+ *
+ * This opens the debugfs directory corresponding to device for use
+ * with igt_sysfs_get() and related functions.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_debugfs_gt_dir(int device, unsigned int gt)
+{
+	int debugfs_gt_dir_fd;
+	char path[PATH_MAX];
+	char gtpath[16];
+	int ret;
+
 	if (!igt_debugfs_path(device, path, sizeof(path)))
 		return -1;
 
-	igt_debug("Opening debugfs directory '%s'\n", path);
-	return open(path, O_RDONLY);
+	ret = snprintf(gtpath, sizeof(gtpath), "/gt%u", gt);
+	igt_assert(ret < sizeof(gtpath));
+	strncat(path, gtpath, sizeof(path) - 1);
+
+	debugfs_gt_dir_fd = open(path, O_RDONLY);
+	igt_debug_on_f(debugfs_gt_dir_fd < 0, "path: %s\n", path);
+
+	return debugfs_gt_dir_fd;
 }
 
 /**
  * igt_debugfs_connector_dir:
  * @device: fd of the device
- * @conn_name: conenctor name
+ * @conn_name: connector name
  * @mode: mode bits as used by open()
  *
  * This opens the debugfs directory corresponding to connector on the device
@@ -267,6 +301,7 @@ int igt_debugfs_pipe_dir(int device, int pipe, int mode)
 
 /**
  * igt_debugfs_open:
+ * @device: fd of the device
  * @filename: name of the debugfs node to open
  * @mode: mode bits as used by open()
  *
@@ -312,8 +347,38 @@ bool igt_debugfs_exists(int device, const char *filename, int mode)
 	return false;
 }
 
+/**
+ * igt_debugfs_gt_open:
+ * @device: open i915 drm fd
+ * @gt: gt instance number
+ * @filename: name of the debugfs node to open
+ * @mode: mode bits as used by open()
+ *
+ * This opens a debugfs file as a Unix file descriptor. The filename should be
+ * relative to the drm device's root, i.e. without "drm/$minor".
+ *
+ * Returns:
+ * The Unix file descriptor for the debugfs file or -1 if that didn't work out.
+ */
+int
+igt_debugfs_gt_open(int device, unsigned int gt, const char *filename, int mode)
+{
+	int dir, ret;
+
+	dir = igt_debugfs_gt_dir(device, gt);
+	if (dir < 0)
+		return dir;
+
+	ret = openat(dir, filename, mode);
+
+	close(dir);
+
+	return ret;
+}
+
 /**
  * igt_debugfs_simple_read:
+ * @dir: fd of the debugfs directory
  * @filename: file name
  * @buf: buffer where the contents will be stored, allocated by the caller
  * @size: size of the buffer
@@ -323,7 +388,7 @@ bool igt_debugfs_exists(int device, const char *filename, int mode)
  * first argument.
  *
  * Returns:
- * -errorno on failure or bytes read on success
+ * -errno on failure or bytes read on success
  */
 int igt_debugfs_simple_read(int dir, const char *filename, char *buf, int size)
 {
@@ -340,6 +405,7 @@ int igt_debugfs_simple_read(int dir, const char *filename, char *buf, int size)
 
 /**
  * __igt_debugfs_read:
+ * @fd: fd of the device
  * @filename: file name
  * @buf: buffer where the contents will be stored, allocated by the caller
  * @size: size of the buffer
@@ -375,6 +441,7 @@ void __igt_debugfs_write(int fd, const char *filename, const char *buf, int size
 
 /**
  * igt_debugfs_search:
+ * @device: fd of the device
  * @filename: file name
  * @substring: string to search for in @filename
  *
@@ -407,9 +474,35 @@ bool igt_debugfs_search(int device, const char *filename, const char *substring)
 	return matched;
 }
 
+/**
+ * igt_ignore_long_hpd:
+ * @drm_fd: the DRM device file fd
+ * @enable: flag to control the ignore long HPD entry
+ *
+ * Set / unset ignore long HPD events from the panels. Some panels
+ * generate long HPDs even while connected to the ports causing
+ * unexpected CI execution issues. Set this to ignore such unexpected
+ * long HPDs where we dont expect to disconnect the displays.
+ */
+bool igt_ignore_long_hpd(int drm_fd, bool enable)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_ignore_long_hpd", O_WRONLY);
+	int ret;
+
+	if (fd < 0) {
+		igt_debug("couldn't open ignore long hpd file\n");
+		return false;
+	}
+
+	ret = write(fd, enable ? "1" : "0", 1);
+
+	close(fd);
+	return ret == 1;
+}
+
 static void igt_hpd_storm_exit_handler(int sig)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = drm_open_driver(DRIVER_INTEL | DRIVER_XE);
 
 	/* Here we assume that only one i915 device will be ever present */
 	igt_hpd_storm_reset(fd);
@@ -419,6 +512,7 @@ static void igt_hpd_storm_exit_handler(int sig)
 
 /**
  * igt_hpd_storm_set_threshold:
+ * @drm_fd: fd of the device
  * @threshold: How many hotplugs per second required to trigger an HPD storm,
  * or 0 to disable storm detection.
  *
@@ -449,6 +543,7 @@ void igt_hpd_storm_set_threshold(int drm_fd, unsigned int threshold)
 
 /**
  * igt_hpd_storm_reset:
+ * @drm_fd: fd of the device
  *
  * Convienence helper to reset HPD storm detection to it's default settings.
  * If hotplug detection was disabled on any ports due to an HPD storm, it will
@@ -476,6 +571,7 @@ void igt_hpd_storm_reset(int drm_fd)
 
 /**
  * igt_hpd_storm_detected:
+ * @drm_fd: fd of the device
  *
  * Checks whether or not i915 has detected an HPD interrupt storm on any of the
  * system's ports.
@@ -515,6 +611,7 @@ bool igt_hpd_storm_detected(int drm_fd)
 
 /**
  * igt_require_hpd_storm_ctl:
+ * @drm_fd: fd of the device
  *
  * Skips the current test if the system does not have HPD storm detection.
  *
@@ -553,6 +650,7 @@ void igt_reset_fifo_underrun_reporting(int drm_fd)
 
 /**
  * igt_drop_caches_has:
+ * @drm_fd: fd of the device
  * @val: bitmask for DROP_* values
  *
  * This queries the debugfs to see if it supports the full set of desired
@@ -573,6 +671,7 @@ bool igt_drop_caches_has(int drm_fd, uint64_t val)
 
 /**
  * igt_drop_caches_set:
+ * @drm_fd: fd of the device
  * @val: bitmask for DROP_* values
  *
  * This calls the debugfs interface the drm/i915 GEM driver exposes to drop or
