@@ -204,35 +204,65 @@ static void untile(int tiled_pos, int x_tile_size, int y_tile_size,
 }
 
 static int linear_x_y_to_xtiled_pos(int x, int y, uint32_t stride, int swizzle,
-				    int bpp)
+				    int bpp, int x_tile_size, int y_tile_size)
 {
 	int pos;
 	int pixel_size = bpp / 8;
 
 	x *= pixel_size;
-	pos = tile(x, y, 512, 8, stride, true);
+	pos = tile(x, y, x_tile_size, y_tile_size, stride, true);
 	pos = swizzle_addr(pos, swizzle);
 	return pos / pixel_size;
 }
 
+static int gen2_linear_x_y_to_xtiled_pos(int x, int y, uint32_t stride, int swizzle,
+					 int bpp)
+{
+	return linear_x_y_to_xtiled_pos(x, y, stride, swizzle, bpp, 128, 16);
+}
+
+static int gen3_linear_x_y_to_xtiled_pos(int x, int y, uint32_t stride, int swizzle,
+					 int bpp)
+{
+	return linear_x_y_to_xtiled_pos(x, y, stride, swizzle, bpp, 512, 8);
+}
+
 static int linear_x_y_to_ytiled_pos(int x, int y, uint32_t stride, int swizzle,
-				    int bpp)
+				    int bpp, int x_tile_size, int y_tile_size,
+				    int ow_size)
 {
 	int ow_tile_n, pos;
-	int ow_size = 16;
 	int pixel_size = bpp / 8;
 
 	/* We have an Y tiling of OWords, so use the tile() function to get the
 	 * OW number, then adjust to the fact that the OW may have more than one
 	 * pixel. */
 	x *= pixel_size;
-	ow_tile_n = tile(x / ow_size, y, 128 / ow_size, 32,
-			 stride / ow_size, false);
+	ow_tile_n = tile(x / ow_size, y, x_tile_size / ow_size,
+			 y_tile_size, stride / ow_size, false);
 	pos = ow_tile_n * ow_size + (x % ow_size);
 	pos = swizzle_addr(pos, swizzle);
 	return pos / pixel_size;
 }
 
+static int gen2_linear_x_y_to_ytiled_pos(int x, int y, uint32_t stride, int swizzle,
+					 int bpp)
+{
+	return linear_x_y_to_ytiled_pos(x, y, stride, swizzle, bpp, 128, 16, 8);
+}
+
+static int i915_linear_x_y_to_ytiled_pos(int x, int y, uint32_t stride, int swizzle,
+					 int bpp)
+{
+	return linear_x_y_to_ytiled_pos(x, y, stride, swizzle, bpp, 512, 8, 32);
+}
+
+static int i945_linear_x_y_to_ytiled_pos(int x, int y, uint32_t stride, int swizzle,
+					 int bpp)
+{
+	return linear_x_y_to_ytiled_pos(x, y, stride, swizzle, bpp, 128, 32, 16);
+}
+
 #define OW_SIZE 16			/* in bytes */
 #define TILE_4_SUBTILE_SIZE 64		/* in bytes */
 #define TILE_4_WIDTH 128		/* in bytes */
@@ -299,32 +329,63 @@ static int linear_x_y_to_4tiled_pos(int x, int y, uint32_t stride, int swizzle,
 }
 
 static void xtiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
-				     int swizzle, int bpp, int *x, int *y)
+				     int swizzle, int bpp, int *x, int *y,
+				     int x_tile_size, int y_tile_size)
 {
 	int pixel_size = bpp / 8;
 
 	tiled_pos = swizzle_addr(tiled_pos, swizzle);
 
-	untile(tiled_pos, 512, 8, stride, true, x, y);
+	untile(tiled_pos, x_tile_size, y_tile_size, stride, true, x, y);
 	*x /= pixel_size;
 }
 
+static void gen2_xtiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
+					  int swizzle, int bpp, int *x, int *y)
+{
+	return xtiled_pos_to_x_y_linear(tiled_pos, stride, swizzle, bpp, x, y, 128, 16);
+}
+
+static void gen3_xtiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
+					  int swizzle, int bpp, int *x, int *y)
+{
+	return xtiled_pos_to_x_y_linear(tiled_pos, stride, swizzle, bpp, x, y, 512, 8);
+}
+
 static void ytiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
-				     int swizzle, int bpp, int *x, int *y)
+				     int swizzle, int bpp, int *x, int *y,
+				     int x_tile_size, int y_tile_size, int ow_size)
 {
 	int ow_tile_n;
-	int ow_size = 16;
 	int pixel_size = bpp / 8;
 
 	tiled_pos = swizzle_addr(tiled_pos, swizzle);
 
 	ow_tile_n = tiled_pos / ow_size;
-	untile(ow_tile_n, 128 / ow_size, 32, stride / ow_size, false, x, y);
+	untile(ow_tile_n, x_tile_size / ow_size, y_tile_size, stride / ow_size, false, x, y);
 	*x *= ow_size;
 	*x += tiled_pos % ow_size;
 	*x /= pixel_size;
 }
 
+static void gen2_ytiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
+					  int swizzle, int bpp, int *x, int *y)
+{
+	ytiled_pos_to_x_y_linear(tiled_pos, stride, swizzle, bpp, x, y, 128, 16, 8);
+}
+
+static void i915_ytiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
+					  int swizzle, int bpp, int *x, int *y)
+{
+	ytiled_pos_to_x_y_linear(tiled_pos, stride, swizzle, bpp, x, y, 512, 8, 32);
+}
+
+static void i945_ytiled_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
+					  int swizzle, int bpp, int *x, int *y)
+{
+	ytiled_pos_to_x_y_linear(tiled_pos, stride, swizzle, bpp, x, y, 128, 32, 16);
+}
+
 static void tile4_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
 				    int swizzle, int bpp, int *x, int *y)
 {
@@ -363,14 +424,20 @@ static void tile4_pos_to_x_y_linear(int tiled_pos, uint32_t stride,
 	*y = tile_origin_y + subtile_origin_y + oword_num;
 }
 
-static void set_pixel(void *_ptr, int index, uint32_t color, int bpp)
+static void set_pixel(void *_ptr, int index, uint64_t color, int bpp)
 {
-	if (bpp == 16) {
+	if (bpp == 8) {
+		uint8_t *ptr = _ptr;
+		ptr[index] = color;
+	} else if (bpp == 16) {
 		uint16_t *ptr = _ptr;
 		ptr[index] = color;
 	} else if (bpp == 32) {
 		uint32_t *ptr = _ptr;
 		ptr[index] = color;
+	} else if (bpp == 64) {
+		uint64_t *ptr = _ptr;
+		ptr[index] = color;
 	} else {
 		igt_assert_f(false, "bpp: %d\n", bpp);
 	}
@@ -403,7 +470,7 @@ static void switch_blt_tiling(struct intel_bb *ibb, uint32_t tiling, bool on)
 }
 
 static void draw_rect_ptr_linear(void *ptr, uint32_t stride,
-				 struct rect *rect, uint32_t color, int bpp)
+				 struct rect *rect, uint64_t color, int bpp)
 {
 	int x, y, line_begin;
 
@@ -414,47 +481,58 @@ static void draw_rect_ptr_linear(void *ptr, uint32_t stride,
 	}
 }
 
-static void draw_rect_ptr_tiled(void *ptr, uint32_t stride, uint32_t tiling,
-				int swizzle, struct rect *rect, uint32_t color,
+typedef int (*linear_x_y_to_tiled_pos_fn)(int x, int y, uint32_t stride, int swizzle,
+					  int bpp);
+
+static linear_x_y_to_tiled_pos_fn linear_to_tiled_fn(int fd, uint32_t tiling)
+{
+	const struct intel_device_info *info =
+		intel_get_device_info(intel_get_drm_devid(fd));
+
+	switch (tiling) {
+	case I915_TILING_X:
+		if (info->graphics_ver == 2)
+			return gen2_linear_x_y_to_xtiled_pos;
+		else
+			return gen3_linear_x_y_to_xtiled_pos;
+	case I915_TILING_Y:
+		if (info->graphics_ver == 2)
+			return gen2_linear_x_y_to_ytiled_pos;
+		else if (info->is_grantsdale || info->is_alviso)
+			return i915_linear_x_y_to_ytiled_pos;
+		else
+			return i945_linear_x_y_to_ytiled_pos;
+	case I915_TILING_4:
+		return linear_x_y_to_4tiled_pos;
+	default:
+		igt_assert(false);
+	}
+}
+
+static void draw_rect_ptr_tiled(int fd, void *ptr, uint32_t stride, uint32_t tiling,
+				int swizzle, struct rect *rect, uint64_t color,
 				int bpp)
 {
+	linear_x_y_to_tiled_pos_fn linear_x_y_to_tiled_pos =
+		linear_to_tiled_fn(fd, tiling);
 	int x, y, pos;
 
 	for (y = rect->y; y < rect->y + rect->h; y++) {
 		for (x = rect->x; x < rect->x + rect->w; x++) {
-			switch (tiling) {
-			case I915_TILING_X:
-				pos = linear_x_y_to_xtiled_pos(x, y, stride,
-							       swizzle, bpp);
-				break;
-			case I915_TILING_Y:
-				pos = linear_x_y_to_ytiled_pos(x, y, stride,
-							       swizzle, bpp);
-				break;
-			case I915_TILING_4:
-				pos = linear_x_y_to_4tiled_pos(x, y, stride,
-							       swizzle, bpp);
-				break;
-			default:
-				igt_assert(false);
-			}
+			pos = linear_x_y_to_tiled_pos(x, y, stride, swizzle, bpp);
 			set_pixel(ptr, pos, color, bpp);
 		}
 	}
 }
 
 static void draw_rect_mmap_cpu(int fd, struct buf_data *buf, struct rect *rect,
-			       uint32_t tiling, uint32_t swizzle, uint32_t color)
+			       uint32_t tiling, uint32_t swizzle, uint64_t color)
 {
-	uint32_t *ptr;
+	void *ptr;
 
 	gem_set_domain(fd, buf->handle, I915_GEM_DOMAIN_CPU,
 		       I915_GEM_DOMAIN_CPU);
 
-	/* We didn't implement suport for the older tiling methods yet. */
-	if (tiling != I915_TILING_NONE)
-		igt_require(intel_display_ver(intel_get_drm_devid(fd)) >= 5);
-
 	ptr = gem_mmap__cpu_coherent(fd, buf->handle, 0, PAGE_ALIGN(buf->size),
 				     PROT_READ | PROT_WRITE);
 
@@ -465,7 +543,7 @@ static void draw_rect_mmap_cpu(int fd, struct buf_data *buf, struct rect *rect,
 	case I915_TILING_X:
 	case I915_TILING_Y:
 	case I915_TILING_4:
-		draw_rect_ptr_tiled(ptr, buf->stride, tiling, swizzle, rect,
+		draw_rect_ptr_tiled(fd, ptr, buf->stride, tiling, swizzle, rect,
 				    color, buf->bpp);
 		break;
 	default:
@@ -479,9 +557,9 @@ static void draw_rect_mmap_cpu(int fd, struct buf_data *buf, struct rect *rect,
 }
 
 static void draw_rect_mmap_gtt(int fd, struct buf_data *buf, struct rect *rect,
-			       uint32_t color)
+			       uint64_t color)
 {
-	uint32_t *ptr;
+	void *ptr;
 
 	gem_set_domain(fd, buf->handle, I915_GEM_DOMAIN_GTT,
 		       I915_GEM_DOMAIN_GTT);
@@ -495,18 +573,14 @@ static void draw_rect_mmap_gtt(int fd, struct buf_data *buf, struct rect *rect,
 }
 
 static void draw_rect_mmap_wc(int fd, struct buf_data *buf, struct rect *rect,
-			      uint32_t tiling, uint32_t swizzle, uint32_t color)
+			      uint32_t tiling, uint32_t swizzle, uint64_t color)
 {
-	uint32_t *ptr;
+	void *ptr;
 
 	if (is_i915_device(fd)) {
 		gem_set_domain(fd, buf->handle, I915_GEM_DOMAIN_GTT,
 			       I915_GEM_DOMAIN_GTT);
 
-		/* We didn't implement suport for the older tiling methods yet. */
-		if (tiling != I915_TILING_NONE)
-			igt_require(intel_display_ver(intel_get_drm_devid(fd)) >= 5);
-
 		if (gem_has_lmem(fd))
 			ptr = gem_mmap_offset__fixed(fd, buf->handle, 0,
 						     PAGE_ALIGN(buf->size),
@@ -530,7 +604,7 @@ static void draw_rect_mmap_wc(int fd, struct buf_data *buf, struct rect *rect,
 	case I915_TILING_X:
 	case I915_TILING_Y:
 	case I915_TILING_4:
-		draw_rect_ptr_tiled(ptr, buf->stride, tiling, swizzle, rect,
+		draw_rect_ptr_tiled(fd, ptr, buf->stride, tiling, swizzle, rect,
 				    color, buf->bpp);
 		break;
 	default:
@@ -542,7 +616,7 @@ static void draw_rect_mmap_wc(int fd, struct buf_data *buf, struct rect *rect,
 }
 
 static void draw_rect_pwrite_untiled(int fd, struct buf_data *buf,
-				     struct rect *rect, uint32_t color)
+				     struct rect *rect, uint64_t color)
 {
 	int i, y, offset;
 	int pixel_size = buf->bpp / 8;
@@ -557,10 +631,40 @@ static void draw_rect_pwrite_untiled(int fd, struct buf_data *buf,
 	}
 }
 
+typedef void (*tiled_pos_to_x_y_linear_fn)(int tiled_pos, uint32_t stride,
+					   int swizzle, int bpp, int *x, int *y);
+
+static tiled_pos_to_x_y_linear_fn tiled_to_linear_fn(int fd, uint32_t tiling)
+{
+	const struct intel_device_info *info =
+		intel_get_device_info(intel_get_drm_devid(fd));
+
+	switch (tiling) {
+	case I915_TILING_X:
+		if (info->graphics_ver == 2)
+			return gen2_xtiled_pos_to_x_y_linear;
+		else
+			return gen3_xtiled_pos_to_x_y_linear;
+	case I915_TILING_Y:
+		if (info->graphics_ver == 2)
+			return gen2_ytiled_pos_to_x_y_linear;
+		else if (info->is_grantsdale || info->is_alviso)
+			return i915_ytiled_pos_to_x_y_linear;
+		else
+			return i945_ytiled_pos_to_x_y_linear;
+	case I915_TILING_4:
+		return tile4_pos_to_x_y_linear;
+	default:
+		igt_assert(false);
+	}
+}
+
 static void draw_rect_pwrite_tiled(int fd, struct buf_data *buf,
 				   uint32_t tiling, struct rect *rect,
-				   uint32_t color, uint32_t swizzle)
+				   uint64_t color, uint32_t swizzle)
 {
+	tiled_pos_to_x_y_linear_fn tiled_pos_to_x_y_linear =
+		tiled_to_linear_fn(fd, tiling);
 	int i;
 	int tiled_pos, x, y, pixel_size;
 	uint8_t tmp[4096];
@@ -569,9 +673,6 @@ static void draw_rect_pwrite_tiled(int fd, struct buf_data *buf,
 	int tmp_start_pos = 0;
 	int pixels_written = 0;
 
-	/* We didn't implement suport for the older tiling methods yet. */
-	igt_require(intel_display_ver(intel_get_drm_devid(fd)) >= 5);
-
 	pixel_size = buf->bpp / 8;
 	tmp_size = sizeof(tmp) / pixel_size;
 
@@ -582,22 +683,8 @@ static void draw_rect_pwrite_tiled(int fd, struct buf_data *buf,
 		set_pixel(tmp, i, color, buf->bpp);
 
 	for (tiled_pos = 0; tiled_pos < buf->size; tiled_pos += pixel_size) {
-		switch (tiling) {
-		case I915_TILING_X:
-			xtiled_pos_to_x_y_linear(tiled_pos, buf->stride,
-						 swizzle, buf->bpp, &x, &y);
-			break;
-		case I915_TILING_Y:
-			ytiled_pos_to_x_y_linear(tiled_pos, buf->stride,
-						 swizzle, buf->bpp, &x, &y);
-			break;
-		case I915_TILING_4:
-			tile4_pos_to_x_y_linear(tiled_pos, buf->stride,
-						swizzle, buf->bpp, &x, &y);
-			break;
-		default:
-			igt_assert(false);
-		}
+		tiled_pos_to_x_y_linear(tiled_pos, buf->stride,
+					swizzle, buf->bpp, &x, &y);
 
 		if (x >= rect->x && x < rect->x + rect->w &&
 		    y >= rect->y && y < rect->y + rect->h) {
@@ -624,7 +711,7 @@ static void draw_rect_pwrite_tiled(int fd, struct buf_data *buf,
 
 static void draw_rect_pwrite(int fd, struct buf_data *buf,
 			     struct rect *rect, uint32_t tiling,
-			     uint32_t swizzle, uint32_t color)
+			     uint32_t swizzle, uint64_t color)
 {
 	switch (tiling) {
 	case I915_TILING_NONE:
@@ -674,7 +761,7 @@ static struct intel_buf *create_buf(int fd, struct buf_ops *bops,
 
 static void draw_rect_blt(int fd, struct cmd_data *cmd_data,
 			  struct buf_data *buf, struct rect *rect,
-			  uint32_t tiling, uint32_t color)
+			  uint32_t tiling, uint64_t color)
 {
 	struct intel_bb *ibb;
 	struct intel_buf *dst;
@@ -702,7 +789,8 @@ static void draw_rect_blt(int fd, struct cmd_data *cmd_data,
 			blt_cmd_depth = 2 << 19;
 			break;
 		case 64:
-			/* Not used or supported yet */
+			blt_cmd_depth = 3 << 19;
+			break;
 		default:
 			igt_assert(false);
 		}
@@ -737,7 +825,7 @@ static void draw_rect_blt(int fd, struct cmd_data *cmd_data,
 					   dst->addr.offset);
 		intel_bb_out(ibb, 0);	/* TODO: Pass down enough info for target memory hint */
 		intel_bb_out(ibb, color);
-		intel_bb_out(ibb, 0);	/* 64 bit color */
+		intel_bb_out(ibb, color >> 32);	/* 64 bit color */
 		intel_bb_out(ibb, 0);	/* 96 bit color */
 		intel_bb_out(ibb, 0);	/* 128 bit color */
 		intel_bb_out(ibb, 0);	/* clear address */
@@ -745,6 +833,30 @@ static void draw_rect_blt(int fd, struct cmd_data *cmd_data,
 		intel_bb_out(ibb, (1 << 29) | ((pitch-1) << 14) | (buf_height-1));
 		intel_bb_out(ibb, 0);	/* mipmap levels / qpitch */
 		intel_bb_out(ibb, 0);	/* mipmap index / alignment */
+	} else if (buf->bpp == 64) {
+		int x = rect->x * 2;
+		int w = rect->w * 2;
+
+		blt_cmd_depth = 3 << 24; /* 32bpp */
+		blt_cmd_len = ((ver >= 8) ?  0x4 : 0x3) + 8*8;
+		blt_cmd_tiling = (tiling) ? XY_COLOR_BLT_TILED : 0;
+		pitch = (ver >= 4 && tiling) ? buf->stride / 4 : buf->stride;
+
+		switch_blt_tiling(ibb, tiling, true);
+
+		intel_bb_out(ibb, XY_PAT_BLT_IMMEDIATE_CMD_NOLEN | XY_COLOR_BLT_WRITE_ALPHA |
+			     XY_COLOR_BLT_WRITE_RGB | blt_cmd_tiling | blt_cmd_len);
+		intel_bb_out(ibb, blt_cmd_depth | (0xF0 << 16) | pitch);
+		intel_bb_out(ibb, (rect->y << 16) | x);
+		intel_bb_out(ibb, ((rect->y + rect->h) << 16) | (x + w));
+		intel_bb_emit_reloc_fenced(ibb, dst->handle, 0, I915_GEM_DOMAIN_RENDER,
+					   0, dst->addr.offset);
+		for (int i = 0; i < 8*8; i += 2) {
+			intel_bb_out(ibb, color);
+			intel_bb_out(ibb, color >> 32);
+		}
+
+		switch_blt_tiling(ibb, tiling, false);
 	} else {
 		switch (buf->bpp) {
 		case 8:
@@ -785,7 +897,7 @@ static void draw_rect_blt(int fd, struct cmd_data *cmd_data,
 
 static void draw_rect_render(int fd, struct cmd_data *cmd_data,
 			     struct buf_data *buf, struct rect *rect,
-			     uint32_t tiling, uint32_t color)
+			     uint32_t tiling, uint64_t color)
 {
 	struct intel_buf *src, *dst;
 	uint32_t devid = intel_get_drm_devid(fd);
@@ -858,7 +970,7 @@ void igt_draw_rect(int fd, struct buf_ops *bops, uint32_t ctx,
 		   int buf_width, int buf_height,
 		   uint32_t tiling, enum igt_draw_method method,
 		   int rect_x, int rect_y, int rect_w, int rect_h,
-		   uint32_t color, int bpp)
+		   uint64_t color, int bpp)
 {
 	uint32_t buf_tiling, swizzle;
 
@@ -932,7 +1044,7 @@ void igt_draw_rect(int fd, struct buf_ops *bops, uint32_t ctx,
 void igt_draw_rect_fb(int fd, struct buf_ops *bops,
 		      uint32_t ctx, struct igt_fb *fb,
 		      enum igt_draw_method method, int rect_x, int rect_y,
-		      int rect_w, int rect_h, uint32_t color)
+		      int rect_w, int rect_h, uint64_t color)
 {
 	igt_draw_rect(fd, bops, ctx, fb->gem_handle, fb->size, fb->strides[0],
 		      fb->width, fb->height,
@@ -949,7 +1061,7 @@ void igt_draw_rect_fb(int fd, struct buf_ops *bops,
  *
  * This function just paints an igt_fb using the provided color.
  */
-void igt_draw_fill_fb(int fd, struct igt_fb *fb, uint32_t color)
+void igt_draw_fill_fb(int fd, struct igt_fb *fb, uint64_t color)
 {
 	igt_draw_rect_fb(fd, NULL, 0, fb,
 			 igt_draw_supports_method(fd, IGT_DRAW_MMAP_GTT) ?
