@@ -40,7 +40,9 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <xe_drm.h>
 
+#include "drmtest.h"
 #include "igt_core.h"
 #include "igt_sysfs.h"
 #include "igt_device.h"
@@ -184,7 +186,7 @@ char *igt_sysfs_path(int device, char *path, int pathlen)
 	snprintf(path, pathlen, "/sys/dev/char/%d:%d",
 		 major(st.st_rdev), minor(st.st_rdev));
 
-	if (access(path, F_OK))
+	if (igt_debug_on(access(path, F_OK)))
 		return NULL;
 
 	return path;
@@ -204,7 +206,7 @@ int igt_sysfs_open(int device)
 {
 	char path[80];
 
-	if (!igt_sysfs_path(device, path, sizeof(path)))
+	if (igt_debug_on(!igt_sysfs_path(device, path, sizeof(path))))
 		return -1;
 
 	return open(path, O_RDONLY);
@@ -263,6 +265,101 @@ int xe_sysfs_gt_open(int xe_device, int gt)
 	return open(path, O_RDONLY);
 }
 
+/**
+ * xe_sysfs_gt_has_node:
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @node: node inside sysfs gt dir that needs to be checked for existence
+ *
+ * This checks if specified node exists in device sysfs gt directory.
+ *
+ * Returns:
+ * true if node exists in sysfs, false otherwise.
+ */
+bool xe_sysfs_gt_has_node(int xe_device, int gt, const char *node)
+{
+	bool has_node;
+	int gt_fd;
+
+	gt_fd = xe_sysfs_gt_open(xe_device, gt);
+	if (gt_fd < 0)
+		return false;
+
+	has_node = igt_sysfs_has_attr(gt_fd, node);
+	close(gt_fd);
+
+	return has_node;
+}
+
+static const char *xe_engine_class_to_str(__u16 class)
+{
+	static const char * const str[] = {
+		[DRM_XE_ENGINE_CLASS_RENDER] = "rcs",
+		[DRM_XE_ENGINE_CLASS_COPY] = "bcs",
+		[DRM_XE_ENGINE_CLASS_VIDEO_DECODE] = "vcs",
+		[DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE] = "vecs",
+		[DRM_XE_ENGINE_CLASS_COMPUTE] = "ccs",
+	};
+
+	if (class < ARRAY_SIZE(str))
+		return str[class];
+
+	return "unk";
+}
+
+/**
+ * xe_sysfs_engine_path:
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ * @path: buffer to fill with the sysfs gt path to the device
+ * @pathlen: length of @path buffer
+ *
+ * Returns:
+ * The directory path, or NULL on failure.
+ */
+char *
+xe_sysfs_engine_path(int xe_device, int gt, int class, char *path, int pathlen)
+{
+	struct stat st;
+	int tile = IS_PONTEVECCHIO(intel_get_drm_devid(xe_device)) ? gt : 0;
+
+	if (xe_device < 0)
+		return NULL;
+
+	if (igt_debug_on(fstat(xe_device, &st)) || igt_debug_on(!S_ISCHR(st.st_mode)))
+		return NULL;
+
+	snprintf(path, pathlen, "/sys/dev/char/%d:%d/device/tile%d/gt%d/engines/%s",
+		 major(st.st_rdev), minor(st.st_rdev), tile, gt, xe_engine_class_to_str(class));
+
+	if (!access(path, F_OK))
+		return path;
+
+	return NULL;
+}
+
+/**
+ * xe_sysfs_engine_open:
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ *
+ * This opens the sysfs gt directory corresponding to device and tile for use
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int xe_sysfs_engine_open(int xe_device, int gt, int class)
+{
+	char path[96];
+
+	if (!xe_sysfs_engine_path(xe_device, gt, class, path, sizeof(path)))
+		return -1;
+
+	return open(path, O_RDONLY);
+}
+
 /**
  * igt_sysfs_gt_path:
  * @device: fd of the device
@@ -341,14 +438,125 @@ int igt_sysfs_get_num_gt(int device)
 	return num_gts;
 }
 
+/**
+ * igt_sysfs_drm_module_params_open:
+ *
+ * This opens the sysfs directory corresponding to drm module
+ * parameters.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_sysfs_drm_module_params_open(void)
+{
+	char path[] = "/sys/module/drm/parameters";
+
+	if (access(path, F_OK))
+		return -1;
+
+	return open(path, O_RDONLY);
+}
+
+static int log_level = -1;
+
+/**
+ * igt_drm_debug_level_get:
+ *
+ * This reads the current debug log level of the machine on
+ * which the test is currently executing.
+ *
+ * Returns:
+ * The current log level, or -1 on error.
+ */
+int igt_drm_debug_level_get(int dir)
+{
+	char buf[20];
+
+	if (log_level >= 0)
+		return log_level;
+
+	if (igt_sysfs_read(dir, "debug", buf, sizeof(buf) - 1) < 0)
+		return -1;
+
+	return atoi(buf);
+}
+
+/**
+ * igt_drm_debug_level_reset:
+ *
+ * This modifies the current debug log level of the machine
+ * to the default value post-test.
+ *
+ */
+void igt_drm_debug_level_reset(void)
+{
+	char buf[20];
+	int dir;
+
+	if (log_level < 0)
+		return;
+
+	dir = igt_sysfs_drm_module_params_open();
+	if (dir < 0)
+		return;
+
+	igt_debug("Resetting DRM debug level to %d\n", log_level);
+	snprintf(buf, sizeof(buf), "%d", log_level);
+	igt_assert(igt_sysfs_set(dir, "debug", buf));
+
+	close(dir);
+}
+
+static void igt_drm_debug_level_reset_exit_handler(int sig)
+{
+	igt_drm_debug_level_reset();
+}
+
+/**
+ * igt_drm_debug_level_update:
+ * @debug_level: new debug level to set
+ *
+ * This modifies the current drm debug log level to the new value.
+ */
+void igt_drm_debug_level_update(unsigned int new_log_level)
+{
+	char buf[20];
+	int dir;
+
+	dir = igt_sysfs_drm_module_params_open();
+	if (dir < 0)
+		return;
+
+	log_level = igt_drm_debug_level_get(dir);
+	if (log_level < 0) {
+		close(dir);
+		return;
+	}
+
+	igt_debug("Setting DRM debug level to %d\n", new_log_level);
+	snprintf(buf, sizeof(buf), "%d", new_log_level);
+	igt_assert(igt_sysfs_set(dir, "debug", buf));
+
+	close(dir);
+
+	/*
+	 * TODO: Check whether multiple exit handlers will get installed,
+	 * if we call this api multiple times
+	 */
+	igt_install_exit_handler(igt_drm_debug_level_reset_exit_handler);
+}
+
 /**
  * igt_sysfs_write:
- * @dir: directory for the device from igt_sysfs_open()
+ * @dir: sysfs directory
  * @attr: name of the sysfs node to open
  * @data: the block to write from
  * @len: the length to write
  *
- * This writes @len bytes from @data to the sysfs file.
+ * This writes @len bytes from @data to the sysfs file.  Contrary to
+ * igt_sysfs_set(), this does not automatically write a null char if len is 0.
+ * It's caller responsibility to pass the right len according to the data being
+ * written.
  *
  * Returns:
  * The number of bytes written, or -errno on error.
@@ -369,7 +577,7 @@ int igt_sysfs_write(int dir, const char *attr, const void *data, int len)
 
 /**
  * igt_sysfs_read:
- * @dir: directory for the device from igt_sysfs_open()
+ * @dir: sysfs directory
  * @attr: name of the sysfs node to open
  * @data: the block to read into
  * @len: the maximum length to read
@@ -395,7 +603,7 @@ int igt_sysfs_read(int dir, const char *attr, void *data, int len)
 
 /**
  * igt_sysfs_set:
- * @dir: directory for the device from igt_sysfs_open()
+ * @dir: sysfs directory
  * @attr: name of the sysfs node to open
  * @value: the string to write
  *
@@ -407,12 +615,20 @@ int igt_sysfs_read(int dir, const char *attr, void *data, int len)
 bool igt_sysfs_set(int dir, const char *attr, const char *value)
 {
 	int len = strlen(value);
+
+	/*
+	 * Always write at least 1 char, the null byte, otherwise it
+	 * won't write anything on sysfs.
+	 */
+	if (!len)
+		len = 1;
+
 	return igt_sysfs_write(dir, attr, value, len) == len;
 }
 
 /**
  * igt_sysfs_get:
- * @dir: directory for the device from igt_sysfs_open()
+ * @dir: sysfs directory
  * @attr: name of the sysfs node to open
  *
  * This reads the value from the sysfs file.
@@ -506,38 +722,59 @@ int igt_sysfs_vprintf(int dir, const char *attr, const char *fmt, va_list ap)
 {
 	char stack[128], *buf = stack;
 	va_list tmp;
-	int ret, fd;
+	int ret, fd, len;
 
 	fd = openat(dir, attr, O_WRONLY);
 	if (igt_debug_on(fd < 0))
 		return -errno;
 
 	va_copy(tmp, ap);
-	ret = vsnprintf(buf, sizeof(stack), fmt, tmp);
+	ret = vsnprintf(stack, sizeof(stack), fmt, tmp);
 	va_end(tmp);
-	if (igt_debug_on(ret < 0))
-		return -EINVAL;
+	if (igt_debug_on(ret < 0)) {
+		ret = -EINVAL;
+		goto end;
+	}
 
-	if (ret > sizeof(stack)) {
-		unsigned int len = ret + 1;
+	len = ret;
 
-		buf = malloc(len);
-		if (igt_debug_on(!buf))
-			return -ENOMEM;
+	if (!ret) {
+		/*
+		 * Make sure to always issue a write() syscall, even if writing
+		 * an empty string, otherwise values in sysfs like module
+		 * parameters don't really get overwritten.  vsnprintf()
+		 * guarantees to return a \0 terminated string, so just add
+		 * that char. The return code is still the same as before, to
+		 * abstract that from caller.
+		 */
+		ret = 1;
+	} else if (ret > sizeof(stack)) {
+		buf = malloc(len + 1);
+		if (igt_debug_on(!buf)) {
+			ret = -ENOMEM;
+			goto end;
+		}
 
-		ret = vsnprintf(buf, ret, fmt, ap);
-		if (igt_debug_on(ret > len)) {
-			free(buf);
-			return -EINVAL;
+		ret = vsnprintf(buf, len + 1, fmt, ap);
+		if (igt_debug_on(ret != len)) {
+			ret = -EINVAL;
+			goto free_buf;
 		}
 	}
 
 	ret = igt_writen(fd, buf, ret);
 
-	close(fd);
+	/* Caller shouldn't know about special sysfs handling, just return 0 */
+	if (!len && ret == 1)
+		ret = 0;
+
+free_buf:
 	if (buf != stack)
 		free(buf);
 
+end:
+	close(fd);
+
 	return ret;
 }
 
@@ -636,6 +873,77 @@ void igt_sysfs_set_u32(int dir, const char *attr, uint32_t value)
 		     "Failed to write %u to %s attribute (%s)\n", value, attr, strerror(errno));
 }
 
+/**
+ * __igt_sysfs_get_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to read
+ * @value: pointer for storing read value
+ *
+ * Convenience wrapper to read a signed 32bit integer from a sysfs file.
+ *
+ * Returns:
+ * True if value successfully read, false otherwise.
+ */
+bool __igt_sysfs_get_s32(int dir, const char *attr, int32_t *value)
+{
+	if (igt_debug_on(igt_sysfs_scanf(dir, attr, "%d", value) != 1))
+		return false;
+
+	return true;
+}
+
+/**
+ * igt_sysfs_get_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to read
+ *
+ * Convenience wrapper to read a signed 32bit integer from a sysfs file.
+ * It asserts on failure.
+ *
+ * Returns:
+ * Read value.
+ */
+int32_t igt_sysfs_get_s32(int dir, const char *attr)
+{
+	int32_t value;
+
+	igt_assert_f(__igt_sysfs_get_s32(dir, attr, &value),
+		     "Failed to read %s attribute (%s)\n", attr, strerror(errno));
+
+	return value;
+}
+
+/**
+ * __igt_sysfs_set_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to write
+ * @value: value to set
+ *
+ * Convenience wrapper to write a signed 32bit integer to a sysfs file.
+ *
+ * Returns:
+ * True if successfully written, false otherwise.
+ */
+bool __igt_sysfs_set_s32(int dir, const char *attr, int32_t value)
+{
+	return igt_sysfs_printf(dir, attr, "%d", value) > 0;
+}
+
+/**
+ * igt_sysfs_set_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to write
+ * @value: value to set
+ *
+ * Convenience wrapper to write a signed 32bit integer to a sysfs file.
+ * It asserts on failure.
+ */
+void igt_sysfs_set_s32(int dir, const char *attr, int32_t value)
+{
+	igt_assert_f(__igt_sysfs_set_s32(dir, attr, value),
+		     "Failed to write %d to %s attribute (%s)\n", value, attr, strerror(errno));
+}
+
 /**
  * __igt_sysfs_get_u64:
  * @dir: directory corresponding to attribute
@@ -856,58 +1164,6 @@ void bind_fbcon(bool enable)
 	bind_con("frame buffer device", enable);
 }
 
-/**
- * kick_snd_hda_intel:
- *
- * This functions unbinds the snd_hda_intel driver so the module cand be
- * unloaded.
- *
- */
-void kick_snd_hda_intel(void)
-{
-	DIR *dir;
-	struct dirent *snd_hda;
-	int fd; size_t len;
-
-	const char *dpath = "/sys/bus/pci/drivers/snd_hda_intel";
-	const char *path = "/sys/bus/pci/drivers/snd_hda_intel/unbind";
-	const char *devid = "0000:";
-
-	fd = open(path, O_WRONLY);
-	if (fd < 0) {
-		return;
-	}
-
-	dir = opendir(dpath);
-	if (!dir)
-		goto out;
-
-	len = strlen(devid);
-	while ((snd_hda = readdir(dir))) {
-		struct stat st;
-		char fpath[PATH_MAX];
-
-		if (*snd_hda->d_name == '.')
-			continue;
-
-		snprintf(fpath, sizeof(fpath), "%s/%s", dpath, snd_hda->d_name);
-		if (lstat(fpath, &st))
-			continue;
-
-		if (!S_ISLNK(st.st_mode))
-			continue;
-
-		if (!strncmp(devid, snd_hda->d_name, len)) {
-			igt_ignore_warn(write(fd, snd_hda->d_name,
-					strlen(snd_hda->d_name)));
-		}
-	}
-
-	closedir(dir);
-out:
-	close(fd);
-}
-
 static int fbcon_cursor_blink_fd = -1;
 static char fbcon_cursor_blink_prev_value[2];
 
@@ -967,7 +1223,7 @@ static int rw_attr_sweep(igt_sysfs_rw_attr_t *rw)
 	while (set < UINT64_MAX / 2) {
 		ret = __igt_sysfs_set_u64(rw->dir, rw->attr, set);
 		__igt_sysfs_get_u64(rw->dir, rw->attr, &get);
-		igt_debug("'%s': ret %d set %lu get %lu\n", rw->attr, ret, set, get);
+		igt_debug("'%s': ret %d set %"PRIu64" get %"PRIu64"\n", rw->attr, ret, set, get);
 		if (ret && rw_attr_equal_within_epsilon(get, set, rw->tol)) {
 			igt_debug("'%s': matches\n", rw->attr);
 			num_points++;
@@ -1011,7 +1267,7 @@ void igt_sysfs_rw_attr_verify(igt_sysfs_rw_attr_t *rw)
 	igt_assert(rw->start);	/* cannot be 0 */
 
 	__igt_sysfs_get_u64(rw->dir, rw->attr, &prev);
-	igt_debug("'%s': prev %lu\n", rw->attr, prev);
+	igt_debug("'%s': prev %"PRIu64"\n", rw->attr, prev);
 
 	ret = rw_attr_sweep(rw);
 
@@ -1025,6 +1281,93 @@ void igt_sysfs_rw_attr_verify(igt_sysfs_rw_attr_t *rw)
 	igt_assert(!ret);
 }
 
+/**
+ * xe_get_engine_class:
+ * @name: de_d_name that we get from igt_sysfs_engine.
+ *
+ * It returns engine class corresponding to the engine dir from igt_sysfs_engines.
+ *
+ */
+static uint16_t xe_get_engine_class(char *name)
+{
+	uint16_t class;
+
+	if (strcmp(name, "rcs") == 0) {
+		class = DRM_XE_ENGINE_CLASS_RENDER;
+	} else if (strcmp(name, "bcs") == 0) {
+		class = DRM_XE_ENGINE_CLASS_COPY;
+	} else if (strcmp(name, "vcs") == 0) {
+		class = DRM_XE_ENGINE_CLASS_VIDEO_DECODE;
+	} else if (strcmp(name, "vecs") == 0) {
+		class = DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE;
+	} else if (strcmp(name, "ccs") == 0) {
+		class = DRM_XE_ENGINE_CLASS_COMPUTE;
+	}
+
+	return class;
+}
+
+/**
+ * igt_sysfs_get_engine_list:
+ * @engines: fd of the directory engine
+ *
+ * Iterates over sysfs/engines and returns an array of
+ * opened engines.  The user will be in charge of closing
+ * the opened engines.
+ *
+ * The returned array will always be terminated by a -1.
+ */
+int *igt_sysfs_get_engine_list(int engines)
+{
+	struct dirent *de;
+	DIR *dir;
+	const int array_max = 16;
+	int *ret = calloc(array_max, sizeof(int));
+	int size = 0;
+
+	igt_assert(ret);
+
+	lseek(engines, 0, SEEK_SET);
+
+	dir = fdopendir(engines);
+	if (!dir)
+		close(engines);
+
+	while ((de = readdir(dir))) {
+		if (*de->d_name == '.')
+			continue;
+		igt_assert_lt(size, array_max);
+		ret[size] = openat(engines, de->d_name, O_RDONLY);
+		if (ret[size] < 0) {
+			ret[size] = 0;
+			continue;
+		}
+		size += 1;
+	}
+
+	igt_assert_lt(size, array_max);
+	ret[size] = -1;
+
+	return ret;
+}
+
+/**
+ * igt_sysfs_free_engine_list:
+ * @list: list of opened engines
+ * @size: number of engines in list
+ *
+ * Helper for cleaning up after igt_sysfs_get_engine_list.
+ * Closes all engines in list before freeing the list.
+ */
+void igt_sysfs_free_engine_list(int *list)
+{
+	int i = 0;
+
+	while (list[i] != -1)
+		close(list[i++]);
+	free(list);
+}
+
 /**
  * igt_sysfs_engines:
  * @xe: fd of the device
@@ -1035,11 +1378,12 @@ void igt_sysfs_rw_attr_verify(igt_sysfs_rw_attr_t *rw)
  * It iterates over sysfs/engines and runs a dynamic engine test.
  *
  */
-void igt_sysfs_engines(int xe, int engines, const char **property,
-		       void (*test)(int, int, const char **))
+void igt_sysfs_engines(int xe, int engines, int gt, bool all, const char **property,
+		       void (*test)(int, int, const char **, uint16_t, int))
 {
 	struct dirent *de;
 	DIR *dir;
+	uint16_t class;
 
 	lseek(engines, 0, SEEK_SET);
 
@@ -1066,7 +1410,13 @@ void igt_sysfs_engines(int xe, int engines, const char **property,
 				igt_require(fstatat(engine_fd, property[2], &st, 0) == 0);
 			}
 			errno = 0;
-			test(xe, engine_fd, property);
+
+			if (all) {
+				class = xe_get_engine_class(de->d_name);
+				test(xe, engine_fd, property, class, gt);
+			} else {
+				test(xe, engine_fd, property, 0, 0);
+			}
 		}
 		close(engine_fd);
 	}
@@ -1142,3 +1492,142 @@ int xe_sysfs_get_num_tiles(int xe_device)
 
 	return num_tiles;
 }
+
+/**
+ * xe_sysfs_driver_do:
+ * @xe_device: fd of the device
+ * @pci_slot: PCI slot of the device
+ * @action: the action to perform through sysfs on the driver
+ *
+ * Use sysfs to perform an action on the driver.
+ *
+ * Returns: fd of the device, which renewed if needed
+ */
+int xe_sysfs_driver_do(int xe_device, char pci_slot[], enum xe_sysfs_driver_action action)
+{
+	int sysfs;
+
+	sysfs = open("/sys/bus/pci/drivers/xe", O_DIRECTORY);
+	igt_assert(sysfs);
+
+	switch(action) {
+	case XE_SYSFS_DRIVER_BIND:
+		igt_assert(igt_sysfs_set(sysfs, "bind", pci_slot));
+		close(sysfs);
+		break;
+	case XE_SYSFS_DRIVER_TRY_BIND:
+		igt_sysfs_set(sysfs, "bind", pci_slot);
+		close(sysfs);
+		break;
+	case XE_SYSFS_DRIVER_UNBIND:
+		igt_assert(igt_sysfs_set(sysfs, "unbind", pci_slot));
+		close(sysfs);
+		break;
+	case XE_SYSFS_DRIVER_REBIND:
+		igt_assert(igt_sysfs_set(sysfs, "unbind", pci_slot));
+
+		/*
+		 * We need to close the client for a proper release, before
+		 * binding back again.
+		 */
+		close(xe_device);
+
+		igt_assert(igt_sysfs_set(sysfs, "bind", pci_slot));
+		close(sysfs);
+
+		/* Renew the client connection */
+		xe_device = drm_open_driver(DRIVER_XE);
+		igt_assert(xe_device);
+
+		break;
+	default:
+		igt_assert(!"missing");
+	}
+
+	return xe_device;
+}
+
+/**
+ * xe_sysfs_engine_class_get_property
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ * @property: property of engine class to retrieve
+ * @value: pointer for storing read value
+ *
+ * Convenience wrapper to get value of given property for given engine class on given gt.
+ *
+ * Returns: true on success, false on failure.
+ */
+bool xe_sysfs_engine_class_get_property(int xe_device, int gt, uint16_t class, const char *property,
+					uint32_t *value)
+{
+	int engines_fd;
+
+	engines_fd = xe_sysfs_engine_open(xe_device, gt, class);
+
+	if (engines_fd == -1) {
+		igt_debug("Failed to open %s on gt%d.\n", xe_engine_class_to_str(class), gt);
+
+		return false;
+	}
+
+	if (!__igt_sysfs_get_u32(engines_fd, property, value)) {
+		igt_debug("Failed to read %s property of %s on gt%d.\n", property,
+			  xe_engine_class_to_str(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	close(engines_fd);
+
+	return true;
+}
+
+/**
+ * xe_sysfs_engine_class_set_property
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ * @property: property of engine class to be modified
+ * @new_value: value to be set
+ * @old_value: pointer for storing old value, can be NULL
+ *
+ * Convenience wrapper to set given property for given engine class on given gt to given value.
+ *
+ * Returns: true on success, false on failure.
+ */
+bool xe_sysfs_engine_class_set_property(int xe_device, int gt, uint16_t class, const char *property,
+					uint32_t new_value, uint32_t *old_value)
+{
+	int engines_fd;
+
+	engines_fd = xe_sysfs_engine_open(xe_device, gt, class);
+
+	if (engines_fd == -1) {
+		igt_debug("Failed to open %s on gt%d.\n", xe_engine_class_to_str(class), gt);
+
+		return false;
+	}
+
+	if (old_value && !__igt_sysfs_get_u32(engines_fd, property, old_value)) {
+		igt_debug("Failed to read %s property of %s on gt%d.\n", property,
+			  xe_engine_class_to_str(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	if (!__igt_sysfs_set_u32(engines_fd, property, new_value)) {
+		igt_debug("Failed to write %s property of %s on gt%d.\n", property,
+			  xe_engine_class_to_str(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	close(engines_fd);
+
+	return true;
+}
