@@ -9,6 +9,10 @@
 #ifndef INTEL_COMPUTE_H
 #define INTEL_COMPUTE_H
 
+#include <stdbool.h>
+
+#include "xe_drm.h"
+
 /*
  * OpenCL Kernels are generated using:
  *
@@ -23,10 +27,16 @@ struct intel_compute_kernels {
 	int ip_ver;
 	unsigned int size;
 	const unsigned char *kernel;
+	unsigned int sip_kernel_size;
+	const unsigned char *sip_kernel;
+	unsigned int long_kernel_size;
+	const unsigned char *long_kernel;
 };
 
 extern const struct intel_compute_kernels intel_compute_square_kernels[];
 
 bool run_intel_compute_kernel(int fd);
-
+bool xe_run_intel_compute_kernel_on_engine(int fd, struct drm_xe_engine_class_instance *eci);
+bool run_intel_compute_kernel_preempt(int fd, struct drm_xe_engine_class_instance *eci,
+				      bool threadgroup_preemption);
 #endif	/* INTEL_COMPUTE_H */
