@@ -26,20 +26,39 @@
  * TEST: kms chamelium color
  * Category: Display
  * Description: Test Color Features at Pipe level using Chamelium to verify instead of CRC
+ * Driver requirement: i915, xe
+ * Functionality: chamelium, colorspace
+ * Mega feature: Color Management
+ * Test category: functionality test
  */
 
 #include "kms_color_helper.h"
 
-IGT_TEST_DESCRIPTION("Test Color Features at Pipe level using Chamelium to verify instead of CRC");
-
 /**
  * SUBTEST: degamma
  * Description: Verify that degamma LUT transformation works correctly
- * Functionality: chamelium, colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- * Driver requirement: i915, xe
+ *
+ * SUBTEST: gamma
+ * Description: Verify that gamma LUT transformation works correctly
+ *
+ * SUBTEST: ctm-%s
+ * Description: Check the color transformation %arg[1]
+ *
+ * arg[1]:
+ *
+ * @0-25:            for 0.25 transparancy
+ * @0-50:            for 0.50 transparancy
+ * @0-75:            for 0.75 transparancy
+ * @blue-to-red:     from blue to red
+ * @green-to-red:    from green to red
+ * @limited-range:   with identity matrix
+ * @max:             for max transparancy
+ * @negative:        for negative transparancy
+ * @red-to-blue:     from red to blue
  */
+
+IGT_TEST_DESCRIPTION("Test Color Features at Pipe level using Chamelium to verify instead of CRC");
+
 /*
  * Draw 3 gradient rectangles in red, green and blue, with a maxed out
  * degamma LUT and verify we have the same frame dump as drawing solid color
@@ -129,14 +148,6 @@ static bool test_pipe_degamma(data_t *data,
 	return ret;
 }
 
-/**
- * SUBTEST: gamma
- * Description: Verify that gamma LUT transformation works correctly
- * Functionality: chamelium, colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- * Driver requirement: i915, xe
- */
 /*
  * Draw 3 gradient rectangles in red, green and blue, with a maxed out
  * gamma LUT and verify we have the same frame dump as drawing solid
@@ -457,6 +468,19 @@ static int test_setup(data_t *data, enum pipe p)
 
 	data->primary = igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_PRIMARY);
 
+	/*
+	 * Prefer to run this test on HDMI connector if its connected, since on DP we
+	 * sometimes face DP FSM issue
+	 */
+        for_each_valid_output_on_pipe(&data->display, p, data->output) {
+                for (i = 0; i < data->port_count; i++) {
+                        if ((data->output->config.connector->connector_type == DRM_MODE_CONNECTOR_HDMIA ||
+			    data->output->config.connector->connector_type == DRM_MODE_CONNECTOR_HDMIB) &&
+			    strcmp(data->output->name, chamelium_port_get_name(data->ports[i])) == 0)
+                                return i;
+                }
+        }
+
 	for_each_valid_output_on_pipe(&data->display, p, data->output) {
 		for (i = 0; i < data->port_count; i++) {
 			if (strcmp(data->output->name,
@@ -487,26 +511,6 @@ run_gamma_degamma_tests_for_pipe(data_t *data, enum pipe p,
 		igt_assert(test_t(data, data->primary, data->ports[port_idx]));
 }
 
-/**
- * SUBTEST: ctm-%s
- * Description: Check the color transformation %arg[1]
- * Functionality: chamelium, colorspace
- * Mega feature: Color mgmt
- * Test category: functionality test
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @0-25:            for 0.25 transparancy
- * @0-50:            for 0.50 transparancy
- * @0-75:            for 0.75 transparancy
- * @blue-to-red:     from blue to red
- * @green-to-red:    from green to red
- * @limited-range:   with identity matrix
- * @max:             for max transparancy
- * @negative:        for negative transparancy
- * @red-to-blue:     from red to blue
- */
 static void
 run_ctm_tests_for_pipe(data_t *data, enum pipe p,
 		       color_t *expected_colors,
@@ -718,6 +722,7 @@ run_tests_for_pipe(data_t *data)
 
 igt_main
 {
+	int i;
 	data_t data = {};
 
 	igt_fixture {
@@ -739,6 +744,13 @@ igt_main
 
 		if (!data.port_count)
 			igt_skip("No ports connected\n");
+		/*
+		 * We don't cause any harm by plugging
+		 * discovered ports, just incase they are not plugged
+		 * we currently skip in test_setup
+		 */
+		for( i = 0; i < data.port_count; i++)
+			chamelium_plug(data.chamelium, data.ports[i]);
 
 		kmstest_set_vt_graphics_mode();
 	}
