@@ -28,6 +28,9 @@
  * TEST: kms chamelium edid
  * Category: Display
  * Description: Testing EDID with a Chamelium board
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ * Test category: functionality test
  */
 
 #include <fcntl.h>
@@ -35,6 +38,7 @@
 #include <string.h>
 #include <stdatomic.h>
 #include <xf86drmMode.h>
+#include "drmtest.h"
 
 #include "config.h"
 #include "igt.h"
@@ -48,7 +52,79 @@
 #include "monitor_edids/hdmi_edids.h"
 #include "monitor_edids/monitor_edids_helper.h"
 
+/**
+ * SUBTEST: dp-edid-read
+ * Description: Make sure the EDID exposed by KMS is the same as the screen's
+ * Functionality: chamelium, dp, edid
+ *
+ * SUBTEST: hdmi-edid-read
+ * Description: Make sure the EDID exposed by KMS is the same as the screen's
+ * Functionality: chamelium, hdmi, edid
+ *
+ * SUBTEST: vga-edid-read
+ * Description: Make sure the EDID exposed by KMS is the same as the screen's
+ * Functionality: chamelium, vga, edid
+ *
+ * SUBTEST: dp-edid-resolution-list
+ * Description: Get an EDID with many modes of different configurations, set
+ *              them on the screen and check the screen resolution matches the
+ *              mode resolution.
+ * Functionality: chamelium, dp, edid
+ *
+ * SUBTEST: dp-mode-timings
+ * Description: For each mode of the IGT base EDID, perform a modeset and check
+ *              the mode detected by the Chamelium receiver matches the mode we
+ *              set
+ * Functionality: chamelium, dp, edid
+ *
+ * SUBTEST: hdmi-mode-timings
+ * Description: For each mode of the IGT base EDID, perform a modeset and check
+ *              the mode detected by the Chamelium receiver matches the mode we
+ *              set
+ * Functionality: chamelium, dp, edid
+ */
+
+/**
+ * SUBTEST: dp-edid-stress-resolution-%s
+ * Description: Stress test the DUT by testing multiple EDIDs, one right after
+ *              the other, and ensure their validity by check the real screen
+ *              resolution vs the advertised mode (%arg[1]) resolution.
+ * Functionality: chamelium, dp, edid
+ *
+ * SUBTEST: hdmi-edid-stress-resolution-%s
+ * Description: Stress test the DUT by testing multiple EDIDs, one right after
+ *              the other, and ensure their validity by check the real screen
+ *              resolution vs the advertised mode (%arg[1]) resolution.
+ * Functionality: chamelium, hdmi, edid
+ *
+ * arg[1]:
+ *
+ * @4k:      4K
+ * @non-4k:  Non 4K
+ */
+
+/**
+ * SUBTEST: dp-edid-change-during-%s
+ * Description: Simulate a screen being unplugged and another screen being
+ *              plugged during suspend, check that a uevent is sent and
+ *              connector status is updated during %arg[1]
+ * Functionality: chamelium, dp, edid
+ *
+ * SUBTEST: hdmi-edid-change-during-%s
+ * Description: Simulate a screen being unplugged and another screen being
+ *              plugged during suspend, check that a uevent is sent and
+ *              connector status is updated during %arg[1]
+ * Functionality: chamelium, dp, edid
+ *
+ * arg[1]:
+ *
+ * @hibernate:    hibernation
+ * @suspend:      suspend
+ */
+
+#define CI_LIMIT 25
 #define MODE_CLOCK_ACCURACY 0.05 /* 5% */
+static bool extended;
 
 static void get_connectors_link_status_failed(chamelium_data_t *data,
 					      bool *link_status_failed)
@@ -145,28 +221,6 @@ static void check_mode(struct chamelium *chamelium, struct chamelium_port *port,
 	igt_assert(video_params.vsync_polarity == mode_vsync_polarity);
 }
 
-/**
- * SUBTEST: dp-edid-read
- * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-edid-read
- * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Test category: functionality test
- * Functionality: chamelium, hdmi_edid
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-edid-read
- * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Test category: functionality test
- * Functionality: chamelium, vga_edid
- * Mega feature: VGA
- * Driver requirement: i915, xe
- */
 static const char igt_custom_edid_type_read_desc[] =
 	"Make sure the EDID exposed by KMS is the same as the screen's";
 static void igt_custom_edid_type_read(chamelium_data_t *data,
@@ -206,30 +260,6 @@ static void igt_custom_edid_type_read(chamelium_data_t *data,
 	drmModeFreeConnector(connector);
 }
 
-/**
- * SUBTEST: dp-edid-stress-resolution-%s
- * Description: Stress test the DUT by testing multiple EDIDs, one right after
- *              the other, and ensure their validity by check the real screen
- *              resolution vs the advertised mode (%arg[1]) resolution.
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-edid-stress-resolution-%s
- * Description: Stress test the DUT by testing multiple EDIDs, one right after
- *              the other, and ensure their validity by check the real screen
- *              resolution vs the advertised mode (%arg[1]) resolution.
- * Test category: functionality test
- * Functionality: chamelium, hdmi_edid
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @4k:      4K
- * @non-4k:  Non 4K
- */
 static const char igt_edid_stress_resolution_desc[] =
 	"Stress test the DUT by testing multiple EDIDs, one right after the other, "
 	"and ensure their validity by check the real screen resolution vs the "
@@ -242,9 +272,16 @@ static void edid_stress_resolution(chamelium_data_t *data,
 	int i;
 	struct chamelium *chamelium = data->chamelium;
 	struct udev_monitor *mon = igt_watch_uevents();
+
 	chamelium_reset_state(&data->display, data->chamelium, port,
 			      data->ports, data->port_count);
 
+	/* TODO: Make changes to check if we can get rid of extended flag and explore
+	 * if there is environment variable in CI when executing with igt_runner
+	 */
+
+	if (!extended && is_intel_device(data->drm_fd))
+		edids_list_len = CI_LIMIT;
 
 	for (i = 0; i < edids_list_len; ++i) {
 		struct chamelium_edid *chamelium_edid;
@@ -299,16 +336,6 @@ static void edid_stress_resolution(chamelium_data_t *data,
 			      data->ports, data->port_count);
 }
 
-/**
- * SUBTEST: dp-edid-resolution-list
- * Description: Get an EDID with many modes of different configurations, set
- *              them on the screen and check the screen resolution matches the
- *              mode resolution.
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- */
 static const char igt_edid_resolution_list_desc[] =
 	"Get an EDID with many modes of different configurations, set them on the screen and check the"
 	" screen resolution matches the mode resolution.";
@@ -372,30 +399,6 @@ static void edid_resolution_list(chamelium_data_t *data,
 	drmModeFreeConnector(connector);
 }
 
-/**
- * SUBTEST: dp-edid-change-during-%s
- * Description: Simulate a screen being unplugged and another screen being
- *              plugged during suspend, check that a uevent is sent and
- *              connector status is updated during %arg[1]
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-edid-change-during-%s
- * Description: Simulate a screen being unplugged and another screen being
- *              plugged during suspend, check that a uevent is sent and
- *              connector status is updated during %arg[1]
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @hibernate:    hibernation
- * @suspend:      suspend
- */
 static const char test_suspend_resume_edid_change_desc[] =
 	"Simulate a screen being unplugged and another screen being plugged "
 	"during suspend, check that a uevent is sent and connector status is "
@@ -448,25 +451,6 @@ static void test_suspend_resume_edid_change(chamelium_data_t *data,
 			    link_status_failed[1][p]);
 }
 
-/**
- * SUBTEST: dp-mode-timings
- * Description: For each mode of the IGT base EDID, perform a modeset and check
- *              the mode detected by the Chamelium receiver matches the mode we
- *              set
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-mode-timings
- * Description: For each mode of the IGT base EDID, perform a modeset and check
- *              the mode detected by the Chamelium receiver matches the mode we
- *              set
- * Test category: functionality test
- * Functionality: chamelium, dp_edid
- * Mega feature: DP
- * Driver requirement: i915, xe
- */
 static const char test_mode_timings_desc[] =
 	"For each mode of the IGT base EDID, perform a modeset and check the "
 	"mode detected by the Chamelium receiver matches the mode we set";
@@ -530,8 +514,22 @@ static void test_mode_timings(chamelium_data_t *data,
 	} while (++i < count_modes);
 }
 
+static int opt_handler(int opt, int opt_index, void *_data)
+{
+	switch (opt) {
+	case 'e':
+		extended = true;
+		break;
+	}
+
+	return IGT_OPT_HANDLER_SUCCESS;
+}
+
+const char *help_str =
+	"  -e \tExtended tests.\n";
+
 IGT_TEST_DESCRIPTION("Testing EDID with a Chamelium board");
-igt_main
+igt_main_args("e", NULL, help_str, opt_handler, NULL)
 {
 	chamelium_data_t data;
 	struct chamelium_port *port;
