@@ -28,11 +28,88 @@
  * TEST: kms chamelium frames
  * Category: Display
  * Description: Tests requiring a Chamelium board
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ * Test category: functionality test
  */
+
 #include "igt_eld.h"
 #include "igt_infoframe.h"
 #include "kms_chamelium_helper.h"
 
+/**
+ * SUBTEST: dp-crc-fast
+ * Description: Pick the first mode of the IGT base EDID, display and capture
+ * 		a few frames, then check captured frames are correct
+ * Functionality: chamelium, dp, frame_integrity
+ *
+ * SUBTEST: hdmi-crc-fast
+ * Description: Pick the first mode of the IGT base EDID, display and capture
+ * 		a few frames, then check captured frames are correct
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * SUBTEST: hdmi-%s-formats
+ * Description: Pick the first mode of the IGT base EDID, display and capture a
+ *              few frames, then check captured frames are correct
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * arg[1]:
+ *
+ * @crc-nonplanar:     CRC with non planar formats
+ * @cmp-planar:        Compare with planar formats
+ */
+
+/**
+ * SUBTEST: vga-frame-dump
+ * Description: For each mode of the IGT base EDID, display and capture a few
+ *              frames, then check captured frames are correct
+ * Functionality: chamelium, vga, frame_integrity
+ *
+ * SUBTEST: dp-crc-%s
+ * Description: For each mode of the IGT base EDID, display and capture a %arg[1]
+ *              frame(s), then check captured frame(s) are correct
+ * Functionality: chamelium, dp, frame_integrity
+ *
+ * SUBTEST: hdmi-crc-%s
+ * Description: For each mode of the IGT base EDID, display and capture a %arg[1]
+ *              frame(s), then check captured frame(s) are correct
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * arg[1]:
+ *
+ * @single:    single
+ * @multiple:  multiple
+ */
+
+/**
+ * SUBTEST: dp-frame-dump
+ * Description: For each mode of the IGT base EDID, display and capture a few
+ *              frames, then download the captured frames and compare them
+ *              bit-by-bit to the sent ones
+ * Functionality: chamelium, dp, frame_integrity
+ *
+ * SUBTEST: hdmi-frame-dump
+ * Description: For each mode of the IGT base EDID, display and capture a few
+ *              frames, then download the captured frames and compare them
+ *              bit-by-bit to the sent ones
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * SUBTEST: hdmi-aspect-ratio
+ * Description: Pick a mode with a picture aspect-ratio, capture AVI InfoFrames
+ *              and check they include the relevant fields
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * SUBTEST: hdmi-%s-planes-random
+ * Description: Setup a few overlay planes with random parameters, capture the
+ *              frame and check it matches the expected output
+ * Functionality: chamelium, hdmi, frame_integrity
+ *
+ * arg[1]:
+ *
+ * @crc:     CRC check
+ * @cmp:     Compare
+ */
+
 #define connector_dynamic_subtest(name__, type__)                   \
 	igt_subtest_with_dynamic(name__)                            \
 	for_each_port(p, port) if (chamelium_port_get_type(port) == \
@@ -539,38 +616,6 @@ static void prepare_randomized_plane(chamelium_data_t *data,
 	igt_remove_fb(data->drm_fd, &pattern_fb);
 }
 
-/**
- * SUBTEST: dp-crc-fast
- * Description: Pick the first mode of the IGT base EDID, display and capture
- * 		a few frames, then check captured frames are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-crc-fast
- * Description: Pick the first mode of the IGT base EDID, display and capture
- * 		a few frames, then check captured frames are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- */
-
-/**
- * SUBTEST: hdmi-%s-formats
- * Description: Pick the first mode of the IGT base EDID, display and capture a
- *              few frames, then check captured frames are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @crc-nonplanar:     CRC with non planar formats
- * @cmp-planar:        Compare with planar formats
- */
 static const char test_display_one_mode_desc[] =
 	"Pick the first mode of the IGT base EDID, display and capture a few "
 	"frames, then check captured frames are correct";
@@ -607,36 +652,6 @@ static void test_display_one_mode(chamelium_data_t *data,
 	drmModeFreeConnector(connector);
 }
 
-/**
- * SUBTEST: vga-frame-dump
- * Description: For each mode of the IGT base EDID, display and capture a few
- *              frames, then check captured frames are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: VGA
- * Driver requirement: i915, xe
- *
- * SUBTEST: dp-crc-%s
- * Description: For each mode of the IGT base EDID, display and capture a %arg[1]
- *              frame(s), then check captured frame(s) are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-crc-%s
- * Description: For each mode of the IGT base EDID, display and capture a %arg[1]
- *              frame(s), then check captured frame(s) are correct
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @single:    single
- * @multiple:  multiple
- */
 static const char test_display_all_modes_desc[] =
 	"For each mode of the IGT base EDID, display and capture a few "
 	"frames, then check captured frames are correct";
@@ -695,25 +710,6 @@ static void test_display_all_modes(chamelium_data_t *data,
 	} while (++i < count_modes);
 }
 
-/**
- * SUBTEST: dp-frame-dump
- * Description: For each mode of the IGT base EDID, display and capture a few
- *              frames, then download the captured frames and compare them
- *              bit-by-bit to the sent ones
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-frame-dump
- * Description: For each mode of the IGT base EDID, display and capture a few
- *              frames, then download the captured frames and compare them
- *              bit-by-bit to the sent ones
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- */
 static const char test_display_frame_dump_desc[] =
 	"For each mode of the IGT base EDID, display and capture a few "
 	"frames, then download the captured frames and compare them "
@@ -782,15 +778,6 @@ static void test_display_frame_dump(chamelium_data_t *data,
 	} while (++i < count_modes);
 }
 
-/**
- * SUBTEST: hdmi-aspect-ratio
- * Description: Pick a mode with a picture aspect-ratio, capture AVI InfoFrames
- *              and check they include the relevant fields
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- */
 static const char test_display_aspect_ratio_desc[] =
 	"Pick a mode with a picture aspect-ratio, capture AVI InfoFrames and "
 	"check they include the relevant fields";
@@ -871,20 +858,6 @@ static void test_display_aspect_ratio(chamelium_data_t *data,
 	drmModeFreeConnector(connector);
 }
 
-/**
- * SUBTEST: hdmi-%s-planes-random
- * Description: Setup a few overlay planes with random parameters, capture the
- *              frame and check it matches the expected output
- * Test category: functionality test
- * Functionality: chamelium, frame_integrity
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @crc:     CRC check
- * @cmp:     Compare
- */
 static const char test_display_planes_random_desc[] =
 	"Setup a few overlay planes with random parameters, capture the frame "
 	"and check it matches the expected output";
@@ -967,7 +940,7 @@ static void test_display_planes_random(chamelium_data_t *data,
 	overlay_planes_count = (rand() % overlay_planes_max) + 1;
 	igt_debug("Using %d overlay planes\n", overlay_planes_count);
 
-	overlay_fbs = calloc(sizeof(struct igt_fb), overlay_planes_count);
+	overlay_fbs = calloc(overlay_planes_count, sizeof(struct igt_fb));
 
 	for (i = 0; i < overlay_planes_count; i++) {
 		struct igt_fb *overlay_fb = &overlay_fbs[i];
