@@ -28,10 +28,149 @@
  * TEST: kms chamelium hpd
  * Category: Display
  * Description: Testing HPD with a Chamelium board
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ * Test category: functionality test
  */
 
 #include "kms_chamelium_helper.h"
 
+/**
+ * SUBTEST: dp-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Functionality: chamelium, hotplug, hdmi
+ *
+ * SUBTEST: vga-hpd-fast
+ * Description: Check that we get uevents and updated connector status on
+ * 		hotplug and unplug
+ * Functionality: chamelium, hotplug, vga
+ *
+ * SUBTEST: dp-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, hdmi
+ *
+ * SUBTEST: vga-hpd
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, vga
+ *
+ * SUBTEST: dp-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, hdmi
+ *
+ * SUBTEST: vga-hpd-%s
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug
+ * Functionality: chamelium, hotplug, vga
+ *
+ * arg[1]:
+ *
+ * @enable-disable-mode:     Toggle the mode
+ * @with-enabled-mode:       Enabling the mode
+ */
+
+/**
+ * SUBTEST: dp-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Functionality: chamelium, hotplug, hdmi
+ *
+ * SUBTEST: vga-hpd-for-each-pipe
+ * Description: Check that we get uevents and updated connector status on
+ *              hotplug and unplug for each pipe with valid output
+ * Functionality: chamelium, hotplug, vga
+ *
+ * SUBTEST: dp-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, dp, hibernation
+ *
+ * SUBTEST: hdmi-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, hdmi, hibernation
+ *
+ * SUBTEST: vga-hpd-after-hibernate
+ * Description: Toggle HPD during Hibernation, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, vga, hibernation
+ *
+ * SUBTEST: dp-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, dp, suspend
+ *
+ * SUBTEST: hdmi-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, hdmi, suspend
+ *
+ * SUBTEST: vga-hpd-after-suspend
+ * Description: Toggle HPD during Suspend, check that uevents are sent and
+ *              connector status is updated
+ * Functionality: chamelium, hotplug, vga, suspend
+ *
+ * SUBTEST: common-hpd-after-suspend
+ * Description: Toggle HPD during suspend on all connectors, check that uevents
+ * 		are sent and connector status is updated
+ * Functionality: chamelium, hotplug, suspend
+ *
+ * SUBTEST: common-hpd-after-hibernate
+ * Description: Toggle HPD during suspend on all connectors, check that uevents
+ *              are sent and connector status is updated
+ * Functionality: chamelium, hotplug, hibernation
+ *
+ * SUBTEST: vga-hpd-without-ddc
+ * Description: Disable DDC on a VGA connector, check we still get a uevent on
+ *              hotplug
+ * Functionality: chamelium, hotplug, vga
+ *
+ * SUBTEST: dp-hpd-storm
+ * Description: Trigger a series of hotplugs in a very small timeframe to
+ *              simulate abad cable, check the kernel falls back to polling
+ *              to avoid a hotplug storm
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd-storm
+ * Description: Trigger a series of hotplugs in a very small timeframe to
+ *              simulate abad cable, check the kernel falls back to polling
+ *              to avoid a hotplug storm
+ * Functionality: chamelium, hotplug, hdmi
+ *
+ * SUBTEST: dp-hpd-storm-disable
+ * Description: Disable HPD storm detection, trigger a storm and check the
+ *              kernel doesn't detect one
+ * Functionality: chamelium, hotplug, dp
+ *
+ * SUBTEST: hdmi-hpd-storm-disable
+ * Description: Disable HPD storm detection, trigger a storm and check the
+ *              kernel doesn't detect one
+ * Functionality: chamelium, hotplug, hdmi
+ */
+
 #define HPD_STORM_PULSE_INTERVAL_DP 100 /* ms */
 #define HPD_STORM_PULSE_INTERVAL_HDMI 200 /* ms */
 
@@ -110,84 +249,6 @@ static void try_suspend_resume_hpd(chamelium_data_t *data,
 	}
 }
 
-/**
- * SUBTEST: dp-hpd-fast
- * Description: Check that we get uevents and updated connector status on
- * 		hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-fast
- * Description: Check that we get uevents and updated connector status on
- * 		hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd-fast
- * Description: Check that we get uevents and updated connector status on
- * 		hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: VGA
- * Driver requirement: i915, xe
- *
- * SUBTEST: dp-hpd
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: VGA
- * Driver requirement: i915, xe
- *
- * SUBTEST: dp-hpd-%s
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-%s
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd-%s
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: VGA
- * Driver requirement: i915, xe
- *
- * arg[1]:
- *
- * @enable-disable-mode:     Toggle the mode
- * @with-enabled-mode:       Enabling the mode
- */
 static const char test_basic_hotplug_desc[] =
 	"Check that we get uevents and updated connector status on "
 	"hotplug and unplug";
@@ -254,31 +315,6 @@ static void test_hotplug(chamelium_data_t *data, struct chamelium_port *port,
 	igt_remove_fb(data->drm_fd, &fb);
 }
 
-/**
- * SUBTEST: dp-hpd-for-each-pipe
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug for each pipe with valid output
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-for-each-pipe
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug for each pipe with valid output
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd-for-each-pipe
- * Description: Check that we get uevents and updated connector status on
- *              hotplug and unplug for each pipe with valid output
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: VGA
- * Driver requirement: i915, xe
- */
 static const char test_hotplug_for_each_pipe_desc[] =
 	"Check that we get uevents and updated connector status on "
 	"hotplug and unplug for each pipe with valid output";
@@ -309,10 +345,9 @@ static void test_hotplug_for_each_pipe(chamelium_data_t *data,
 		output = chamelium_get_output_for_port(data, port);
 
 		/* If pipe is valid for output then set it */
-		if (igt_pipe_connector_valid(pipe, output)) {
-			igt_output_set_pipe(output, pipe);
+		igt_output_set_pipe(output, pipe);
+		if (intel_pipe_output_combo_valid(&data->display))
 			igt_display_commit2(&data->display, COMMIT_ATOMIC);
-		}
 
 		chamelium_unplug(data->chamelium, port);
 		chamelium_wait_for_connector_after_hotplug(
@@ -324,55 +359,6 @@ static void test_hotplug_for_each_pipe(chamelium_data_t *data,
 	igt_hpd_storm_reset(data->drm_fd);
 }
 
-/**
- * SUBTEST: dp-hpd-after-hibernate
- * Description: Toggle HPD during Hibernation, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, hibernation
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-after-hibernate
- * Description: Toggle HPD during Hibernation, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, hibernation
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd-after-hibernate
- * Description: Toggle HPD during Hibernation, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, hibernation
- * Mega feature: VGA
- * Driver requirement: i915, xe
- *
- * SUBTEST: dp-hpd-after-suspend
- * Description: Toggle HPD during Suspend, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, suspend
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-after-suspend
- * Description: Toggle HPD during Suspend, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, suspend
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- *
- * SUBTEST: vga-hpd-after-suspend
- * Description: Toggle HPD during Suspend, check that uevents are sent and
- *              connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, suspend
- * Mega feature: VGA
- * Driver requirement: i915, xe
- */
 static const char test_suspend_resume_hpd_desc[] =
 	"Toggle HPD during suspend, check that uevents are sent and connector "
 	"status is updated";
@@ -396,23 +382,6 @@ static void test_suspend_resume_hpd(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
-/**
- * SUBTEST: common-hpd-after-suspend
- * Description: Toggle HPD during suspend on all connectors, check that uevents
- * 		are sent and connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, suspend
- * Mega feature: General Display Features
- * Driver requirement: i915, xe
- *
- * SUBTEST: common-hpd-after-hibernate
- * Description: Toggle HPD during suspend on all connectors, check that uevents
- *              are sent and connector status is updated
- * Test category: functionality test
- * Functionality: chamelium, hotplug, hibernation
- * Mega feature: General Display Features
- * Driver requirement: i915, xe
- */
 static const char test_suspend_resume_hpd_common_desc[] =
 	"Toggle HPD during suspend on all connectors, check that uevents are "
 	"sent and connector status is updated";
@@ -442,15 +411,6 @@ static void test_suspend_resume_hpd_common(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
-/**
- * SUBTEST: vga-hpd-without-ddc
- * Description: Disable DDC on a VGA connector, check we still get a uevent on
- *              hotplug
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: VGA
- * Driver requirement: i915, xe
- */
 static const char test_hpd_without_ddc_desc[] =
 	"Disable DDC on a VGA connector, check we still get a uevent on hotplug";
 static void test_hpd_without_ddc(chamelium_data_t *data,
@@ -477,25 +437,6 @@ static void test_hpd_without_ddc(chamelium_data_t *data,
 	igt_cleanup_uevents(mon);
 }
 
-/**
- * SUBTEST: dp-hpd-storm
- * Description: Trigger a series of hotplugs in a very small timeframe to
- *              simulate abad cable, check the kernel falls back to polling
- *              to avoid a hotplug storm
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-storm
- * Description: Trigger a series of hotplugs in a very small timeframe to
- *              simulate abad cable, check the kernel falls back to polling
- *              to avoid a hotplug storm
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- */
 static const char test_hpd_storm_detect_desc[] =
 	"Trigger a series of hotplugs in a very small timeframe to simulate a"
 	"bad cable, check the kernel falls back to polling to avoid a hotplug "
@@ -530,23 +471,6 @@ static void test_hpd_storm_detect(chamelium_data_t *data,
 	igt_hpd_storm_reset(data->drm_fd);
 }
 
-/**
- * SUBTEST: dp-hpd-storm-disable
- * Description: Disable HPD storm detection, trigger a storm and check the
- *              kernel doesn't detect one
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: DP
- * Driver requirement: i915, xe
- *
- * SUBTEST: hdmi-hpd-storm-disable
- * Description: Disable HPD storm detection, trigger a storm and check the
- *              kernel doesn't detect one
- * Test category: functionality test
- * Functionality: chamelium, hotplug
- * Mega feature: HDMI
- * Driver requirement: i915, xe
- */
 static const char test_hpd_storm_disable_desc[] =
 	"Disable HPD storm detection, trigger a storm and check the kernel "
 	"doesn't detect one";
