@@ -42,66 +42,44 @@
 /**
  * TEST: core hotunplug
  * Description: Examine behavior of a driver on device hot unplug
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: driver
+ * Functionality: device hotplug
  * Feature: SMI, core
  * Test category: functionality test
  *
  * SUBTEST: hotrebind
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description:
- *   Check if the driver can be cleanly rebound to a device with a still open hot unbound driver
- *   instance
- * Run type: FULL
+ * Description: Check if the driver can be cleanly rebound to a device with a
+ *		still open hot unbound driver instance
  *
  * SUBTEST: hotrebind-lateclose
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if a hot unbound driver instance still open after hot rebind can be cleanly released
- * Run type: FULL
+ * Description: Check if a hot unbound driver instance still open after hot
+ *		rebind can be cleanly released
  *
  * SUBTEST: hotreplug
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if a hot unplugged and still open device can be cleanly restored
- * Run type: FULL
+ * Description: Check if a hot unplugged and still open device can be cleanly
+ *		restored
  *
  * SUBTEST: hotreplug-lateclose
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if an instance of a still open while hot replugged device can be cleanly released
- * Run type: FULL
+ * Description: Check if an instance of a still open while hot replugged device
+ *		can be cleanly released
  *
  * SUBTEST: hotunbind-rebind
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if the driver can be cleanly unbound from an open device, then released and rebound
- * Run type: FULL
+ * Description: Check if the driver can be cleanly unbound from an open device,
+ *		then released and rebound
  *
  * SUBTEST: hotunplug-rescan
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if an open device can be cleanly unplugged, then released and restored
- * Run type: FULL
+ * Description: Check if an open device can be cleanly unplugged, then released
+ *		and restored
  *
  * SUBTEST: unbind-rebind
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if the driver can be cleanly unbound from a device believed to be closed, then rebound
- * Run type: BAT
+ * Description: Check if the driver can be cleanly unbound from a device
+ *		believed to be closed, then rebound
  *
  * SUBTEST: unplug-rescan
- * Category: Software building block
- * Sub-category: driver
- * Functionality: device hotplug
- * Description: Check if a device believed to be closed can be cleanly unplugged, then restored
- * Run type: FULL
+ * Description: Check if a device believed to be closed can be cleanly
+ *		unplugged, then restored
  */
 
 IGT_TEST_DESCRIPTION("Examine behavior of a driver on device hot unplug");
@@ -164,7 +142,11 @@ static int close_device(int fd_drm, const char *when, const char *which)
 		return fd_drm;
 
 	local_debug("%sclosing %sdevice instance\n", when, which);
-	return local_close(fd_drm, "Device close failed");
+	errno = 0;
+	if (igt_warn_on_f(__drm_close_driver(fd_drm), "Device close failed\n"))
+		return -errno;	/* (never -1) */
+
+	return -1;	/* success */
 }
 
 static int close_sysfs(int fd_sysfs_dev)
@@ -502,7 +484,7 @@ static void node_healthcheck(struct hotunplug *priv, unsigned flags)
 
 static bool healthcheck(struct hotunplug *priv, bool recover)
 {
-	/* give udev some time to recreate device nodes before we continue */
+	/* give devtmpfs some time to recreate device nodes before we continue */
 	sleep(1);
 
 	/* device name may have changed, rebuild IGT device list */
@@ -703,13 +685,18 @@ igt_main
 
 	igt_fixture {
 		int fd_drm;
+		char driver[32] = {};
 
 		fd_drm = __drm_open_driver(DRIVER_ANY);
 		igt_skip_on_f(fd_drm < 0, "No known DRM device found\n");
 
-		if (is_i915_device(fd_drm)) {
-			priv.chipset = DRIVER_INTEL;
+		priv.chipset = drm_get_chipset(fd_drm);
+		/* Make sure it opens the same driver */
+		__get_drm_device_name(fd_drm, driver, sizeof(driver) - 1);
+		__set_forced_driver(driver);
+		igt_info("DRM driver: %s\n", driver);
 
+		if (is_i915_device(fd_drm)) {
 			gem_quiescent_gpu(fd_drm);
 			igt_require_gem(fd_drm);
 
@@ -721,6 +708,10 @@ igt_main
 
 		igt_assert_eq(close_device(fd_drm, "", "selected "), -1);
 
+		fd_drm = __drm_open_driver_render(priv.chipset);
+		igt_skip_on_f(fd_drm < 0, "No known DRM render device found\n");
+		igt_assert_eq(close_device(fd_drm, "", "selected render "), -1);
+
 		prepare(&priv);
 	}
 
