@@ -33,9 +33,9 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
+#include <poll.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/poll.h>
 #include <sys/time.h>
 #include <time.h>
 
@@ -50,61 +50,41 @@
 
 /**
  * TEST: gem exec nop
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: CMD submission
+ * Functionality: execbuf
+ * Feature: cmd_submission
+ * Test category: GEM_Legacy
  *
  * SUBTEST: basic-parallel
- * Category: Infrastructure
- * Feature: cmd_submission
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: basic-sequential
- * Category: Infrastructure
- * Feature: cmd_submission
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: basic-series
- * Category: Infrastructure
- * Feature: cmd_submission
- * Functionality: command submission
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: context-sequential
- * Feature: cmd_submission
  *
  * SUBTEST: forked-sequential
- * Feature: cmd_submission
  *
  * SUBTEST: headless
- * Feature: cmd_submission
  *
  * SUBTEST: independent
- * Feature: cmd_submission
  *
  * SUBTEST: multiple
- * Feature: cmd_submission
  *
  * SUBTEST: parallel
- * Feature: cmd_submission
  *
  * SUBTEST: poll
- * Feature: cmd_submission
  *
  * SUBTEST: poll-sequential
- * Feature: cmd_submission
  *
  * SUBTEST: preempt
  * Feature: cmd_submission, multitile
  *
  * SUBTEST: sequential
- * Feature: cmd_submission
  *
  * SUBTEST: series
- * Feature: cmd_submission
  *
  * SUBTEST: signal
  * Feature: cmd_submission, multitile
@@ -113,7 +93,6 @@
  * Feature: cmd_submission, multitile
  *
  * SUBTEST: single
- * Feature: cmd_submission
  */
 
 #define ENGINE_FLAGS  (I915_EXEC_RING_MASK | I915_EXEC_BSD_MASK)
@@ -176,7 +155,6 @@ static void poll_ring(int fd, const intel_ctx_t *ctx,
 		      int timeout)
 {
 	const unsigned int gen = intel_gen(intel_get_drm_devid(fd));
-	const uint32_t MI_ARB_CHK = 0x5 << 23;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj;
 	struct drm_i915_gem_relocation_entry reloc[4], *r;
@@ -187,7 +165,7 @@ static void poll_ring(int fd, const intel_ctx_t *ctx,
 	uint64_t elapsed;
 
 	flags = I915_EXEC_NO_RELOC;
-	if (gen == 4 || gen == 5)
+	if (gem_store_dword_needs_secure(fd))
 		flags |= I915_EXEC_SECURE;
 
 	igt_require(gem_class_can_store_dword(fd, e->class));
@@ -228,7 +206,7 @@ static void poll_ring(int fd, const intel_ctx_t *ctx,
 
 		b = batch + (start_offset + 64) / sizeof(*batch);
 		bbe[start_offset != 0] = b;
-		*b++ = MI_ARB_CHK;
+		*b++ = MI_ARB_CHECK;
 
 		r->target_handle = obj.handle;
 		r->offset = (b - batch + 1) * sizeof(uint32_t);
@@ -262,7 +240,7 @@ static void poll_ring(int fd, const intel_ctx_t *ctx,
 	do {
 		unsigned int idx = ++cycles & 1;
 
-		*bbe[idx] = MI_ARB_CHK;
+		*bbe[idx] = MI_ARB_CHECK;
 		execbuf.batch_start_offset =
 			(bbe[idx] - batch) * sizeof(*batch) - 64;
 
@@ -289,7 +267,6 @@ static void poll_sequential(int fd, const intel_ctx_t *ctx,
 {
 	const unsigned int gen = intel_gen(intel_get_drm_devid(fd));
 	const struct intel_execution_engine2 *e;
-	const uint32_t MI_ARB_CHK = 0x5 << 23;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj[2];
 	struct drm_i915_gem_relocation_entry reloc[4], *r;
@@ -301,7 +278,7 @@ static void poll_sequential(int fd, const intel_ctx_t *ctx,
 	bool cached;
 
 	flags = I915_EXEC_NO_RELOC;
-	if (gen == 4 || gen == 5)
+	if (gem_store_dword_needs_secure(fd))
 		flags |= I915_EXEC_SECURE;
 
 	nengine = 0;
@@ -356,7 +333,7 @@ static void poll_sequential(int fd, const intel_ctx_t *ctx,
 
 		b = batch + (start_offset + 64) / sizeof(*batch);
 		bbe[start_offset != 0] = b;
-		*b++ = MI_ARB_CHK;
+		*b++ = MI_ARB_CHECK;
 
 		r->target_handle = obj[1].handle;
 		r->offset = (b - batch + 1) * sizeof(uint32_t);
@@ -393,7 +370,7 @@ static void poll_sequential(int fd, const intel_ctx_t *ctx,
 	do {
 		unsigned int idx = ++cycles & 1;
 
-		*bbe[idx] = MI_ARB_CHK;
+		*bbe[idx] = MI_ARB_CHECK;
 		execbuf.batch_start_offset =
 			(bbe[idx] - batch) * sizeof(*batch) - 64;
 
@@ -438,7 +415,7 @@ stable_nop_on_ring(int fd, uint32_t handle, const intel_ctx_t *ctx,
 	igt_stats_t s;
 	double n;
 
-	igt_assert(reps >= 5);
+	igt_assert_lte(5, reps);
 
 	igt_stats_init_with_size(&s, reps);
 	s.is_float = true;
