@@ -20,10 +20,14 @@
 #include "i915/gem.h"
 /**
  * TEST: i915 pipe stress
- * Description:
- *   Stress test how gpu and cpu behaves if maximum amount of planes, cpu and gpu utilization is
- *   achieved in order to reveal possible bandwidth/watermark and similar problems.
- * Run type: FULL
+ * Description: Stress test how gpu and cpu behaves if maximum amount of
+ *              planes, cpu and gpu utilization is achieved in order to reveal
+ *              possible bandwidth/watermark and similar problems.
+ * Category: Display
+ * Mega feature: General Display Features
+ * Sub-category: Display Tests
+ * Functionality: kms_core
+ * Test category: stress test
  *
  * SUBTEST: stress-xrgb8888-untiled
  * Description:
@@ -192,7 +196,6 @@ static struct intel_buf *
 create_buf(struct data *data, int width, int height, uint32_t region)
 {
 	struct intel_buf *buf;
-	uint32_t handle;
 
 	buf = calloc(1, sizeof(*buf));
 	igt_assert(buf);
@@ -201,10 +204,9 @@ create_buf(struct data *data, int width, int height, uint32_t region)
 	 * Legacy code uses 32 bpp after buffer creation.
 	 * Let's do the same due to keep shader intact.
 	 */
-	handle = gem_create_in_memory_regions(data->drm_fd, width * height, region);
-	intel_buf_init_using_handle(data->bops, handle, buf,
-				    width/4, height, 32, 0,
-				    I915_TILING_NONE, 0);
+	intel_buf_init_in_region(data->bops, buf,
+				 width/4, height, 32, 0,
+				 I915_TILING_NONE, 0, region);
 
 	return buf;
 }
@@ -241,7 +243,6 @@ static void *gpu_load(void *ptr)
 	int frame_height;
 	drmModeModeInfo *mode;
 	int frame = 0;
-	int x, y;
 	int rect = 0, total_rects = 0;
 	int pixels = 0;
 
@@ -273,8 +274,8 @@ static void *gpu_load(void *ptr)
 			x_rand = hars_petruska_f54_1_random_unsafe_max(frame_width - rect_width);
 			y_rand = hars_petruska_f54_1_random_unsafe_max(frame_height/2 - rect_height);
 
-			context->blt_rect.x = x + x_rand;
-			context->blt_rect.y = y + y_rand;
+			context->blt_rect.x = x_rand;
+			context->blt_rect.y = y_rand;
 
 			/* Fill randomly sized and positioned rectangles */
 			fill_gpu(context, context->blt_rect.x, context->blt_rect.y,
