@@ -35,6 +35,7 @@
  */
 
 #include "igt.h"
+#include "igt_halffloat.h"
 
 #include "i915/gem_create.h"
 #include "intel_pat.h"
@@ -95,6 +96,7 @@
  * arg[1]:
  *
  * @crc-primary-basic:           Primary plane CRC compatibility
+ * @crc-primary-suspend:         Primary plane CRC after suspend
  * @crc-sprite-planes-basic:     Sprite plane CRC compatability
  * @random-ccs-data:             Random CCS data
  *
@@ -158,6 +160,7 @@ enum test_flags {
 	TEST_BAD_AUX_STRIDE		= 1 << 7,
 	TEST_RANDOM			= 1 << 8,
 	TEST_ALL_PLANES			= 1 << 9,
+	TEST_SUSPEND			= 1 << 10,
 };
 
 #define TEST_BAD_CCS_PLANE	(TEST_NO_AUX_BUFFER | TEST_BAD_CCS_HANDLE | \
@@ -185,6 +188,15 @@ typedef struct {
 	uint64_t ccs_modifier;
 	unsigned int seed;
 	bool user_seed;
+	enum igt_commit_style commit;
+	int fb_list_length;
+	struct {
+		struct igt_fb fb;
+		int width, height;
+		double r, g, b;
+		u64 modifier;
+		u32 format;
+	} *fb_list;
 } data_t;
 
 static const struct {
@@ -200,6 +212,7 @@ static const uint32_t formats[] = {
 	DRM_FORMAT_XYUV8888,
 	DRM_FORMAT_XRGB8888,
 	DRM_FORMAT_XRGB2101010,
+	DRM_FORMAT_XBGR16161616F,
 	DRM_FORMAT_YUYV,
 	DRM_FORMAT_NV12,
 	DRM_FORMAT_P012,
@@ -241,6 +254,12 @@ static const struct {
 	{TEST_BAD_CCS_HANDLE, "ccs-on-another-bo", "Test CCS with different BO with given modifier"},
 	{TEST_BAD_AUX_STRIDE, "bad-aux-stride", "Test with bad AUX stride with given CCS modifier"},
 	{TEST_CRC | TEST_ALL_PLANES, "crc-sprite-planes-basic", "Test sprite plane CRC compatibility with given CCS modifier"},
+
+	/*
+	 * suspend test has to be kept last because it will decompress
+	 * framebuffers when run on BMG
+	 */
+	{TEST_CRC | TEST_SUSPEND, "crc-primary-suspend", "Test primary plane CRC with suspend on XR24 and P016 formats"},
 };
 
 /*
@@ -280,13 +299,6 @@ create_fb_prepare_add(int drm_fd, int width, int height,
 	addfb_init(fb, f);
 }
 
-static bool is_ccs_cc_modifier(uint64_t modifier)
-{
-	return modifier == I915_FORMAT_MOD_Y_TILED_GEN12_RC_CCS_CC ||
-	       modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC ||
-	       modifier == I915_FORMAT_MOD_4_TILED_MTL_RC_CCS_CC;
-}
-
 /*
  * The CCS planes of compressed framebuffers contain non-zero bytes if the
  * engine compressed effectively the framebuffer. The actual encoding of these
@@ -331,7 +343,8 @@ static void check_ccs_cc_plane(int drm_fd, igt_fb_t *fb, int plane, const float
 		uint32_t d;
 	} *cc_p;
 	void *map;
-	uint32_t native_color;
+	uint32_t native_color[2] = {};
+	uint16_t half[4];
 
 	if (is_i915_device(drm_fd)) {
 		gem_set_domain(drm_fd, fb->gem_handle, I915_GEM_DOMAIN_CPU, 0);
@@ -346,15 +359,34 @@ static void check_ccs_cc_plane(int drm_fd, igt_fb_t *fb, int plane, const float
 		   cc_color[2] == cc_p[2].f &&
 		   cc_color[3] == cc_p[3].f);
 
-	native_color = (uint8_t)(cc_color[3] * 0xff) << 24 |
-		       (uint8_t)(cc_color[0] * 0xff) << 16 |
-		       (uint8_t)(cc_color[1] * 0xff) << 8 |
-		       (uint8_t)(cc_color[2] * 0xff);
+	switch (fb->drm_format) {
+	case DRM_FORMAT_XRGB8888:
+		native_color[0] = (uint32_t)(cc_color[3] * 0xff) << 24 |
+			(uint32_t)(cc_color[0] * 0xff) << 16 |
+			(uint32_t)(cc_color[1] * 0xff) << 8 |
+			(uint32_t)(cc_color[2] * 0xff);
+		break;
+	case DRM_FORMAT_XRGB2101010:
+		native_color[0] = (uint32_t)(cc_color[3] * 0x3) << 30 |
+			(uint32_t)(cc_color[0] * 0x3ff) << 20 |
+			(uint32_t)(cc_color[1] * 0x3ff) << 10 |
+			(uint32_t)(cc_color[2] * 0x3ff);
+		break;
+	case DRM_FORMAT_XBGR16161616F:
+		igt_float_to_half(cc_color, half, 4);
 
-	igt_assert_eq_u32(native_color, cc_p[4].d);
+		native_color[1] = (uint64_t)half[3] << 16 | (uint64_t)half[2];
+		native_color[0] = (uint64_t)half[1] << 16 | (uint64_t)half[0];
+		break;
+	default:
+		break;
+	}
+
+	igt_assert_eq_u32(native_color[0], cc_p[4].d);
+	igt_assert_eq_u32(native_color[1], cc_p[5].d);
 
 	igt_assert_eq(0, gem_munmap(map, fb->size));
-};
+}
 
 static void check_all_ccs_planes(int drm_fd, igt_fb_t *fb, const float *cc_color, bool check_cc_plane)
 {
@@ -379,8 +411,8 @@ static void access_flat_ccs_surface(struct igt_fb *fb, bool verify_compression)
 	uint16_t cpu_caching = DRM_XE_GEM_CPU_CACHING_WC;
 	uint8_t uc_mocs = intel_get_uc_mocs_index(fb->fd);
 	uint8_t comp_pat_index = intel_get_pat_idx_wt(fb->fd);
-	uint32_t region = (AT_LEAST_GEN(intel_get_drm_devid(fb->fd), 20) &
-						xe_has_vram(fb->fd)) ? REGION_LMEM(0) : REGION_SMEM;
+	uint32_t region = (intel_gen(intel_get_drm_devid(fb->fd)) >= 20 &&
+			   xe_has_vram(fb->fd)) ? REGION_LMEM(0) : REGION_SMEM;
 
 	struct drm_xe_engine_class_instance inst = {
 		.engine_class = DRM_XE_ENGINE_CLASS_COPY,
@@ -459,7 +491,7 @@ static void fill_fb_random(int drm_fd, igt_fb_t *fb)
 	igt_assert_eq(0, gem_munmap(map, fb->size));
 
 	/* randomize also ccs surface on Xe2 */
-	if (AT_LEAST_GEN(intel_get_drm_devid(drm_fd), 20))
+	if (intel_gen(intel_get_drm_devid(drm_fd)) >= 20)
 		access_flat_ccs_surface(fb, false);
 }
 
@@ -584,144 +616,74 @@ static void fast_clear_fb(int drm_fd, struct igt_fb *fb, const float *cc_color)
 	buf_ops_destroy(bops);
 }
 
-static struct blt_copy_object *blt_fb_init(const struct igt_fb *fb,
-					   uint32_t plane, uint32_t memregion,
-					   uint8_t pat_index)
+static struct igt_fb *get_fb(data_t *data, u64 modifier, double r, double g,
+			     double b, int width, int height, u32 format)
 {
-	uint32_t name, handle;
-	struct blt_copy_object *blt;
-	uint64_t stride;
-
-	blt = malloc(sizeof(*blt));
-	igt_assert(blt);
-
-	name = gem_flink(fb->fd, fb->gem_handle);
-	handle = gem_open(fb->fd, name);
-
-	stride = fb->strides[plane] / 4;
-
-	blt_set_object(blt, handle, fb->size, memregion,
-		       intel_get_uc_mocs_index(fb->fd),
-		       pat_index,
-		       T_TILE4,
-		       COMPRESSION_DISABLED,
-		       COMPRESSION_TYPE_3D);
-
-	blt_set_geom(blt, stride, 0, 0, fb->width, fb->plane_height[plane], 0, 0);
-	blt->plane_offset = fb->offsets[plane];
-	blt->ptr = xe_bo_mmap_ext(fb->fd, handle, fb->size,
-				  PROT_READ | PROT_WRITE);
-	return blt;
-}
-
-static enum blt_color_depth blt_get_bpp(const struct igt_fb *fb)
-{
-	switch (fb->plane_bpp[0]) {
-	case 8:
-		return CD_8bit;
-	case 16:
-		return CD_16bit;
-	case 32:
-		return CD_32bit;
-	case 64:
-		return CD_64bit;
-	case 96:
-		return CD_96bit;
-	case 128:
-		return CD_128bit;
-	default:
-		igt_assert(0);
-	}
-}
-
-static uint32_t blt_compression_format(struct blt_copy_data *blt,
-				       const struct igt_fb *fb)
-{
-	switch (fb->drm_format) {
-	case DRM_FORMAT_XRGB8888:
-	case DRM_FORMAT_XRGB2101010:
-	case DRM_FORMAT_P010:
-	case DRM_FORMAT_P012:
-	case DRM_FORMAT_P016:
-	case DRM_FORMAT_YUYV:
-		return 8;
-	case DRM_FORMAT_XYUV8888:
-	case DRM_FORMAT_NV12:
-		return 9;
-	default:
-		igt_assert_f(0, "Unknown format\n");
-	}
-}
-
-static void xe2_ccs_blit(data_t *data, struct igt_fb *fb, struct igt_fb *temp_fb)
-{
-	uint64_t ahnd = 0;
-
-	struct blt_copy_data blt = {};
-	struct blt_copy_object *src, *dst;
-	struct blt_block_copy_data_ext ext = {}, *pext = NULL;
-	uint32_t mem_region;
-	intel_ctx_t *xe_ctx;
-	uint32_t vm, exec_queue;
-	uint32_t xe_bb;
-	uint64_t bb_size = 4096;
-	struct igt_fb *dst_fb = fb, *src_fb = temp_fb;
-
-	struct drm_xe_engine_class_instance inst = {
-		.engine_class = DRM_XE_ENGINE_CLASS_COPY,
-	};
-
-	vm = xe_vm_create(src_fb->fd, 0, 0);
-
-	exec_queue = xe_exec_queue_create(src_fb->fd, vm, &inst, 0);
-	xe_ctx = intel_ctx_xe(src_fb->fd, vm, exec_queue, 0, 0, 0);
-	mem_region = vram_if_possible(src_fb->fd, 0);
-
-	ahnd = intel_allocator_open_full(src_fb->fd, xe_ctx->vm, 0, 0,
-						INTEL_ALLOCATOR_SIMPLE,
-						ALLOC_STRATEGY_LOW_TO_HIGH, 0);
-
-	bb_size = ALIGN(bb_size + xe_cs_prefetch_size(src_fb->fd),
-			xe_get_default_alignment(src_fb->fd));
-	xe_bb = xe_bo_create(src_fb->fd, 0, bb_size,
-			     vram_if_possible(dst_fb->fd, 0), 0);
-
-	for (int i = 0; i < dst_fb->num_planes; i++) {
-		src = blt_fb_init(src_fb, i, mem_region, intel_get_pat_idx_uc(src_fb->fd));
-		dst = blt_fb_init(dst_fb, i, mem_region, intel_get_pat_idx_wt(dst_fb->fd));
-
-		blt_copy_init(src_fb->fd, &blt);
-		blt.color_depth = blt_get_bpp(src_fb);
-		blt_set_copy_object(&blt.src, src);
-		blt_set_copy_object(&blt.dst, dst);
-
-		blt_set_object_ext(&ext.src,
-				blt_compression_format(&blt, src_fb),
-				src_fb->width, src_fb->height,
-				SURFACE_TYPE_2D);
-
-		blt_set_object_ext(&ext.dst,
-				blt_compression_format(&blt, dst_fb),
-				dst_fb->width, dst_fb->height,
-				SURFACE_TYPE_2D);
-
-		pext = &ext;
-
-		blt_set_batch(&blt.bb, xe_bb, bb_size, mem_region);
-
-		blt_block_copy(src_fb->fd, xe_ctx, NULL, ahnd, &blt, pext);
-
-		blt_destroy_object(src_fb->fd, src);
-		blt_destroy_object(dst_fb->fd, dst);
+	for (int i = 0; i < data->fb_list_length; i++) {
+		if (data->fb_list[i].width == width &&
+		    data->fb_list[i].height == height &&
+		    data->fb_list[i].modifier == modifier &&
+		    data->fb_list[i].format == format &&
+		    data->fb_list[i].r == r && data->fb_list[i].g == g &&
+		    data->fb_list[i].b == b)
+			return &data->fb_list[i].fb;
 	}
 
-	put_ahnd(ahnd);
-	gem_close(dst_fb->fd, xe_bb);
-	xe_exec_queue_destroy(dst_fb->fd, exec_queue);
-	xe_vm_destroy(dst_fb->fd, vm);
-	free(xe_ctx);
+	data->fb_list = realloc(data->fb_list, sizeof(*data->fb_list) *
+				(data->fb_list_length + 1));
+	data->fb_list[data->fb_list_length].width = width;
+	data->fb_list[data->fb_list_length].height = height;
+	data->fb_list[data->fb_list_length].r = r;
+	data->fb_list[data->fb_list_length].g = g;
+	data->fb_list[data->fb_list_length].b = b;
+	data->fb_list[data->fb_list_length].modifier = modifier;
+	data->fb_list[data->fb_list_length].format = format;
 
-	access_flat_ccs_surface(fb, true);
+	if (modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
+	    modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS) {
+		struct igt_fb *temp_fb, *fb;
+		/* copy xe2 framebuffer to compresssed memory
+		 */
+
+		igt_create_fb(data->drm_fd, width, height, format, modifier,
+			      &data->fb_list[data->fb_list_length++].fb);
+
+		/* temp fb non compressed linear fb */
+		temp_fb = get_fb(data, DRM_FORMAT_MOD_NONE, r, g, b,
+				 width, height, data->format);
+
+		/* because of possible realloc happening get 'current' fb
+		 * back from the list
+		 */
+		fb = get_fb(data, modifier, r, g, b, width, height,
+			    data->format);
+
+		igt_xe2_blit_with_dst_pat(fb, temp_fb, intel_get_pat_idx_wt(fb->fd));
+		access_flat_ccs_surface(fb, true);
+		return fb;
+
+	} else if (r + g + b == 0)
+		igt_create_pattern_fb(data->drm_fd, width, height, format,
+				      modifier,
+				      &data->fb_list[data->fb_list_length].fb);
+	else
+		igt_create_color_fb(data->drm_fd, width, height, format,
+				    modifier, r, g, b,
+				    &data->fb_list[data->fb_list_length].fb);
+
+	return &data->fb_list[data->fb_list_length++].fb;
+}
+
+static void remove_compressed_bmg_fbs(data_t *data) {
+	/* if running on bmg ccs become uncompressed at suspend. Here marked
+	 * bmg ccs in book keeping as not there hence it will not match to
+	 * anything. Here cannot remove fb yet since it's on screen. Remove
+	 * will happen at final fixture.
+	 */
+	for (int i = 0; i < data->fb_list_length; i++) {
+		if (data->fb_list[i].modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS)
+			data->fb_list[i].modifier = ~1;
+	}
 }
 
 static void generate_fb(data_t *data, struct igt_fb *fb,
@@ -730,7 +692,10 @@ static void generate_fb(data_t *data, struct igt_fb *fb,
 {
 	struct drm_mode_fb_cmd2 f = {0};
 	uint64_t modifier;
-	cairo_t *cr;
+	struct igt_fb *temp_fb;
+	bool do_fast_clear = igt_fb_is_gen12_rc_ccs_cc_modifier(data->ccs_modifier);
+	bool do_solid_fill = do_fast_clear || data->plane;
+	int c = !!data->plane;
 	const float cc_color[4] = {colors[!!data->plane].r,
 				   colors[!!data->plane].g,
 				   colors[!!data->plane].b,
@@ -746,57 +711,37 @@ static void generate_fb(data_t *data, struct igt_fb *fb,
 	else
 		modifier = DRM_FORMAT_MOD_LINEAR;
 
-	create_fb_prepare_add(data->drm_fd, width, height,
-			      data->format, modifier,
-			      fb, &f);
+	if (data->flags & TEST_RANDOM) {
+		create_fb_prepare_add(data->drm_fd, width, height,
+				data->format, modifier,
+				fb, &f);
+		do_ioctl(data->drm_fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		fb->fb_id = f.fb_id;
+	} else {
+		if (do_solid_fill)
+			temp_fb = get_fb(data, modifier,
+					colors[c].r, colors[c].g, colors[c].b,
+					width, height, data->format);
+		else
+			temp_fb = get_fb(data, modifier, 0.0, 0.0, 0.0,
+					width, height, data->format);
+
+		*fb = *temp_fb;
+		addfb_init(fb, &f);
+	}
 
 	if (data->flags & TEST_RANDOM) {
 		srand(data->seed);
 		fill_fb_random(data->drm_fd, fb);
 	} else {
-		bool do_fast_clear = is_ccs_cc_modifier(data->ccs_modifier);
-		bool do_solid_fill = do_fast_clear || data->plane;
-		int c = !!data->plane;
-
 		if (do_fast_clear && (fb_flags & FB_COMPRESSED)) {
 			fast_clear_fb(data->drm_fd, fb, cc_color);
-		} else {
-			if (modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
-			    modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS) {
-				struct igt_fb temp_fb;
-				// non compressed temporary pattern image
-				if (do_solid_fill)
-					igt_create_color_fb(data->drm_fd, width, height,
-						fb->drm_format, I915_FORMAT_MOD_4_TILED,
-						colors[c].r, colors[c].g, colors[c].b,
-						&temp_fb);
-				else
-					igt_create_pattern_fb(data->drm_fd, width, height,
-							fb->drm_format, I915_FORMAT_MOD_4_TILED,
-							&temp_fb);
-
-				xe2_ccs_blit(data, fb, &temp_fb);
-				igt_remove_fb(data->drm_fd, &temp_fb);
-			} else {
-				cr = igt_get_cairo_ctx(data->drm_fd, fb);
-
-				if (do_solid_fill)
-					igt_paint_color(cr, 0, 0, width, height,
-							colors[c].r, colors[c].g, colors[c].b);
-				else
-					igt_paint_test_pattern(cr, width, height);
-
-				igt_put_cairo_ctx(cr);
-			}
 		}
 	}
 
-	do_ioctl(data->drm_fd, DRM_IOCTL_MODE_ADDFB2, &f);
-
 	if (check_ccs_planes)
 		check_all_ccs_planes(data->drm_fd, fb, cc_color, !(data->flags & TEST_RANDOM));
 
-	fb->fb_id = f.fb_id;
 }
 
 static igt_plane_t *first_sdr_plane(data_t *data)
@@ -830,16 +775,10 @@ static bool try_config(data_t *data, enum test_fb_flags fb_flags,
 	igt_plane_t *primary = compatible_main_plane(data);
 	drmModeModeInfo *drm_mode = igt_output_get_mode(data->output);
 	int fb_width = drm_mode->hdisplay;
-	enum igt_commit_style commit;
 	struct igt_fb fb = {};
 	struct igt_fb fb_sprite = {};
 	int ret;
 
-	if (data->display.is_atomic)
-		commit = COMMIT_ATOMIC;
-	else
-		commit = COMMIT_UNIVERSAL;
-
 	if (primary == data->plane)
 		return false;
 
@@ -847,9 +786,15 @@ static bool try_config(data_t *data, enum test_fb_flags fb_flags,
 				      data->ccs_modifier))
 		return false;
 
-	if (is_ccs_cc_modifier(data->ccs_modifier) &&
+	if (igt_fb_is_gen12_rc_ccs_cc_modifier(data->ccs_modifier) &&
 	    data->format != DRM_FORMAT_XRGB8888 &&
-	    data->format != DRM_FORMAT_XRGB2101010)
+	    data->format != DRM_FORMAT_XRGB2101010 &&
+	    data->format != DRM_FORMAT_XBGR16161616F)
+		return false;
+
+	/* VEBOX just hangs with an actual 10bpc format */
+	if (igt_fb_is_gen12_mc_ccs_modifier(data->ccs_modifier) &&
+	    data->format == DRM_FORMAT_XRGB2101010)
 		return false;
 
 	if ((fb_flags & FB_MISALIGN_AUX_STRIDE) ||
@@ -890,10 +835,24 @@ static bool try_config(data_t *data, enum test_fb_flags fb_flags,
 	if (data->flags & TEST_BAD_ROTATION_90)
 		igt_plane_set_rotation(primary, IGT_ROTATION_90);
 
-	ret = igt_display_try_commit2(display, commit);
+	ret = igt_display_try_commit2(display, data->commit);
 
-	if (ret == 0 && !(fb_flags & TEST_BAD_ROTATION_90) && crc)
+	if (ret == 0 && !(fb_flags & TEST_BAD_ROTATION_90) && crc) {
+		if (data->flags & TEST_SUSPEND && fb_flags & FB_COMPRESSED) {
+			igt_system_suspend_autoresume(SUSPEND_STATE_MEM,
+						      SUSPEND_TEST_NONE);
+
+			/* on resume check flat ccs is still compressed */
+			if (is_xe_device(data->drm_fd) &&
+			    HAS_FLATCCS(intel_get_drm_devid(data->drm_fd))) {
+				if (IS_BATTLEMAGE(intel_get_drm_devid(data->drm_fd)))
+					remove_compressed_bmg_fbs(data);
+				else
+					access_flat_ccs_surface(&fb, true);
+			}
+		}
 		igt_pipe_crc_collect_crc(data->pipe_crc, crc);
+	}
 
 	igt_debug_wait_for_keypress("ccs");
 
@@ -905,10 +864,11 @@ static bool try_config(data_t *data, enum test_fb_flags fb_flags,
 
 	igt_plane_set_fb(primary, NULL);
 	igt_plane_set_rotation(primary, IGT_ROTATION_0);
-	igt_display_commit2(display, commit);
 
-	igt_remove_fb(data->drm_fd, &fb_sprite);
-	igt_remove_fb(data->drm_fd, &fb);
+	if (data->flags & TEST_RANDOM) {
+		igt_display_commit2(display, data->commit);
+		igt_remove_fb(data->drm_fd, &fb);
+	}
 
 	igt_assert_eq(ret, data->flags & TEST_BAD_ROTATION_90 ? -EINVAL : 0);
 
@@ -920,6 +880,14 @@ static int test_ccs(data_t *data)
 	igt_crc_t crc, ref_crc;
 	enum test_fb_flags fb_flags = 0;
 
+	if (data->flags & TEST_SUSPEND &&
+	    !(data->format == DRM_FORMAT_XRGB8888 ||
+	      data->format == DRM_FORMAT_P016))
+	      return 0;
+
+	igt_info("Testing format " IGT_FORMAT_FMT " / modifier " IGT_MODIFIER_FMT "\n",
+		 IGT_FORMAT_ARGS(data->format), IGT_MODIFIER_ARGS(data->ccs_modifier));
+
 	if (data->flags & TEST_CRC) {
 		data->pipe_crc = igt_pipe_crc_new(data->drm_fd, data->pipe,
 						  IGT_PIPE_CRC_SOURCE_AUTO);
@@ -953,6 +921,45 @@ static int test_ccs(data_t *data)
 	return valid_tests;
 }
 
+static bool skip_plane(data_t *data, igt_plane_t *plane)
+{
+	int index = plane->index;
+
+	if (intel_display_ver(intel_get_drm_devid(data->drm_fd)) < 11)
+		return false;
+
+	/*
+	 * Test 1 HDR plane, 1 SDR UV plane, 1 SDR Y plane.
+	 *
+	 * Kernel registers planes in the hardware Z order:
+	 * 0,1,2 HDR planes
+	 * 3,4 SDR UV planes
+	 * 5,6 SDR Y planes
+	 */
+	return index != 0 && index != 3 && index != 5;
+}
+
+static bool valid_modifier_test(u64 modifier, const enum test_flags flags)
+{
+	switch (modifier) {
+	case I915_FORMAT_MOD_4_TILED_DG2_RC_CCS:
+	case I915_FORMAT_MOD_4_TILED_DG2_MC_CCS:
+	case I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC:
+		if (flags & TEST_BAD_CCS_PLANE)
+			return false;
+		break;
+	case I915_FORMAT_MOD_4_TILED_BMG_CCS:
+	case I915_FORMAT_MOD_4_TILED_LNL_CCS:
+		if (flags & TEST_FAIL_ON_ADDFB2)
+			return false;
+		break;
+	default:
+		break;
+	}
+
+	return true;
+}
+
 static void test_output(data_t *data, const int testnum)
 {
 	uint16_t dev_id;
@@ -963,13 +970,8 @@ static void test_output(data_t *data, const int testnum)
 	data->flags = tests[testnum].flags;
 
 	for (int i = 0; i < ARRAY_SIZE(ccs_modifiers); i++) {
-		if (((ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS ||
-		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_MC_CCS ||
-		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_DG2_RC_CCS_CC) &&
-		    tests[testnum].flags & TEST_BAD_CCS_PLANE) ||
-		    (tests[testnum].flags & TEST_FAIL_ON_ADDFB2 &&
-		    (ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
-		    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS)))
+		if (!valid_modifier_test(ccs_modifiers[i].modifier,
+					 data->flags))
 			continue;
 
 		data->ccs_modifier = ccs_modifiers[i].modifier;
@@ -978,10 +980,10 @@ static void test_output(data_t *data, const int testnum)
 		igt_subtest_with_dynamic_f("%s-%s", tests[testnum].testname, ccs_modifiers[i].str) {
 			if (ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_BMG_CCS ||
 			    ccs_modifiers[i].modifier == I915_FORMAT_MOD_4_TILED_LNL_CCS) {
-				igt_require_f(AT_LEAST_GEN(dev_id, 20),
+				igt_require_f(intel_gen(dev_id) >= 20,
 					      "Xe2 platform needed.\n");
 			} else {
-				igt_require_f(intel_get_device_info(dev_id)->graphics_ver < 20,
+				igt_require_f(intel_gen(dev_id) < 20,
 					      "Older than Xe2 platform needed.\n");
 			}
 
@@ -1003,6 +1005,9 @@ static void test_output(data_t *data, const int testnum)
 						igt_display_require_output_on_pipe(&data->display, data->pipe);
 
 						for_each_plane_on_pipe(&data->display, data->pipe, data->plane) {
+							if (skip_plane(data, data->plane))
+								continue;
+
 							for (int j = 0; j < ARRAY_SIZE(formats); j++) {
 								data->format = formats[j];
 								valid_tests += test_ccs(data);
@@ -1067,12 +1072,22 @@ igt_main_args("cs:", NULL, help_str, opt_handler, &data)
 
 		if (!data.user_seed)
 			data.seed = time(NULL);
+
+		data.commit = data.display.is_atomic ? COMMIT_ATOMIC :
+			COMMIT_UNIVERSAL;
+
+		data.fb_list_length = 0;
 	}
 
 	for (int c = 0; c < ARRAY_SIZE(tests); c++)
 		test_output(&data, c);
 
 	igt_fixture {
+		igt_display_commit2(&data.display, data.commit);
+		for (int i = 0; i < data.fb_list_length; i++)
+			igt_remove_fb(data.drm_fd, &data.fb_list[i].fb);
+		free(data.fb_list);
+
 		igt_display_fini(&data.display);
 		drm_close_driver(data.drm_fd);
 	}
