@@ -22,13 +22,6 @@
  *
  */
 
-#include "igt.h"
-#include <errno.h>
-#include <limits.h>
-#include <stdbool.h>
-#include <stdio.h>
-#include <string.h>
-
 /**
  * TEST: kms cursor crc
  * Category: Display
@@ -40,6 +33,71 @@
  *              compares it with the CRC value obtained when the cursor plane
  *              was disabled and its drawing is directly inserted on the PF by
  *              software.
+ * Driver requirement: i915, xe
+ * Functionality: cursor
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
+#include "igt.h"
+#include <errno.h>
+#include <limits.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <string.h>
+
+/**
+ * SUBTEST: cursor-dpms
+ * Description: Check random placement of a cursor with DPMS.
+ * Functionality: cursor, dpms
+ *
+ * SUBTEST: cursor-suspend
+ * Description: Check random placement of a cursor with suspend.
+ * Functionality: cursor, suspend
+ *
+ * SUBTEST: cursor-size-hints
+ * Description: Check that sizes declared in SIZE_HINTS are accepted.
+ * Functionality: cursor
+ *
+ * SUBTEST: cursor-%s
+ * Description: %arg[1]
+ *
+ * arg[1]:
+ *
+ * @alpha-opaque:       Validates the composition of a fully opaque cursor plane,
+ *                      i.e., alpha channel equal to 1.0.
+ * @alpha-transparent:  Validates the composition of a fully transparent cursor
+ *                      plane, i.e., alpha channel equal to 0.0.
+ * @size-change:        Create a maximum size cursor, then change the size in
+ *                      flight to smaller ones to see that the size is applied
+ *                      correctly.
+ */
+
+/**
+ * SUBTEST: cursor-%s-%s
+ * Description: Check if a %arg[2] cursor is %arg[1].
+ *
+ * arg[1]:
+ *
+ * @offscreen:             well-positioned outside the screen
+ * @onscreen:              well-positioned inside the screen
+ * @random:                randomly placed
+ * @rapid-movement:        rapidly udates for movements
+ * @sliding:               smooth for horizontal, vertical & diagonal movements
+ *
+ * arg[2]:
+ *
+ * @128x128:               128x128 size
+ * @128x42:                128x42 size
+ * @256x256:               256x256 size
+ * @256x85:                256x85 size
+ * @32x10:                 32x10 size
+ * @32x32:                 32x32 size
+ * @512x170:               512x170 size
+ * @512x512:               512x512 size
+ * @64x21:                 64x21 size
+ * @64x64:                 64x64 size
+ * @max-size:              Max supported size
  */
 
 IGT_TEST_DESCRIPTION(
@@ -276,6 +334,15 @@ static void do_single_test(data_t *data, int x, int y, bool hw_test,
 		restore_image(data, swbufidx, &((cursorarea){x, y, data->curw, data->curh}));
 		igt_plane_set_fb(data->primary, &data->primary_fb[swbufidx]);
 		igt_display_commit(display);
+
+		/* Wait for two more vblanks since cursor updates may not
+		 * synchronized to the same frame on AMD HW
+		 */
+		if (is_amdgpu_device(data->drm_fd))
+			igt_wait_for_vblank_count(data->drm_fd,
+				display->pipes[data->pipe].crtc_offset,
+				data->vblank_wait_count);
+
 		igt_pipe_crc_get_current(data->drm_fd, pipe_crc, &crc);
 		igt_assert_crc_equal(&crc, hwcrc);
 	}
@@ -560,14 +627,18 @@ static void create_cursor_fb(data_t *data, int cur_w, int cur_h)
 {
 	cairo_t *cr;
 	uint32_t fb_id;
+	int cur_h_extra_line = 1;
 
+	/* Cropping is not supported for cursor plane by AMD */
+	if (is_amdgpu_device(data->drm_fd))
+		cur_h_extra_line = 0;
 	/*
 	 * Make the FB slightly taller and leave the extra
 	 * line opaque white, so that we can see that the
 	 * hardware won't scan beyond what it should (esp.
 	 * with non-square cursors).
 	 */
-	fb_id = igt_create_color_fb(data->drm_fd, cur_w, cur_h + 1,
+	fb_id = igt_create_color_fb(data->drm_fd, cur_w, cur_h + cur_h_extra_line,
 				    DRM_FORMAT_ARGB8888,
 				    DRM_FORMAT_MOD_LINEAR,
 				    1.0, 1.0, 1.0,
@@ -609,7 +680,7 @@ static void test_cursor_opaque(data_t *data)
 	test_cursor_alpha(data);
 }
 
-static bool require_cursor_size(data_t *data, int w, int h)
+static bool cursor_size_supported(data_t *data, int w, int h)
 {
 	igt_fb_t primary_fb;
 	drmModeModeInfo *mode;
@@ -651,41 +722,9 @@ static bool require_cursor_size(data_t *data, int w, int h)
 	igt_remove_fb(data->drm_fd, &primary_fb);
 	igt_output_set_pipe(output, PIPE_NONE);
 
-	return !!ret;
+	return ret == 0;
 }
 
-/**
- * SUBTEST: cursor-dpms
- * Description: Check random placement of a cursor with DPMS.
- * Driver requirement: i915, xe
- * Functionality: cursor, dpms
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: cursor-suspend
- * Description: Check random placement of a cursor with suspend.
- * Driver requirement: i915, xe
- * Functionality: cursor, suspend
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: cursor-%s
- * Description: %arg[1]
- * Driver requirement: i915, xe
- * Functionality: cursor
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @alpha-opaque:       Validates the composition of a fully opaque cursor plane,
- *                      i.e., alpha channel equal to 1.0.
- * @alpha-transparent:  Validates the composition of a fully transparent cursor
- *                      plane, i.e., alpha channel equal to 0.0.
- * @size-change:        Create a maximum size cursor, then change the size in
- *                      flight to smaller ones to see that the size is applied
- *                      correctly.
- */
 static void run_test(data_t *data, void (*testfunc)(data_t *), int cursor_w, int cursor_h)
 {
 	prepare_crtc(data, cursor_w, cursor_h);
@@ -760,7 +799,7 @@ static bool valid_pipe_output_combo(data_t *data)
 	igt_display_reset(display);
 	igt_output_set_pipe(data->output, data->pipe);
 
-	if (i915_pipe_output_combo_valid(display))
+	if (intel_pipe_output_combo_valid(display))
 		ret = true;
 
 	igt_output_set_pipe(data->output, PIPE_NONE);
@@ -782,37 +821,52 @@ static bool execution_constraint(enum pipe pipe)
 	return false;
 }
 
+static void test_size_hints(data_t *data)
+{
+	const struct drm_plane_size_hint *hints;
+	drmModePropertyBlobPtr blob;
+	uint64_t blob_id;
+	int count;
+
+	igt_require(igt_plane_has_prop(data->cursor, IGT_PLANE_SIZE_HINTS));
+
+	blob_id = igt_plane_get_prop(data->cursor, IGT_PLANE_SIZE_HINTS);
+	/*
+	 * blob_id==0 is reserved for potential future use, but the
+	 * meaning has not yet been defined so fail outright if we see it.
+	 */
+	igt_assert(blob_id);
+
+	blob = drmModeGetPropertyBlob(data->drm_fd, blob_id);
+	igt_assert(blob);
+
+	hints = blob->data;
+	count = blob->length / sizeof(hints[0]);
+	igt_assert_lt(0, count);
+
+	for (int i = 0; i < count; i++) {
+		int w = hints[i].width;
+		int h = hints[i].height;
+
+		igt_create_fb(data->drm_fd, w, h,
+			      DRM_FORMAT_ARGB8888,
+			      DRM_FORMAT_MOD_LINEAR,
+			      &data->fb);
+
+		/*
+		 * TODO
+		 * This only confirms the kernel accepts a cursor
+		 * of this size. Verify that the cusrsor also works
+		 * correctly, if not already covered by other subtests.
+		 */
+		igt_assert(cursor_size_supported(data, w, h));
+
+		igt_remove_fb(data->drm_fd, &data->fb);
+	}
+
+	drmModeFreePropertyBlob(blob);
+}
 
-/**
- * SUBTEST: cursor-%s-%s
- * Description: Check if a %arg[2] cursor is %arg[1].
- * Driver requirement: i915, xe
- * Functionality: cursor
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @offscreen:             well-positioned outside the screen
- * @onscreen:              well-positioned inside the screen
- * @random:                randomly placed
- * @rapid-movement:        rapidly udates for movements
- * @sliding:               smooth for horizontal, vertical & diagonal movements
- *
- * arg[2]:
- *
- * @128x128:               128x128 size
- * @128x42:                128x42 size
- * @256x256:               256x256 size
- * @256x85:                256x85 size
- * @32x10:                 32x10 size
- * @32x32:                 32x32 size
- * @512x170:               512x170 size
- * @512x512:               512x512 size
- * @64x21:                 64x21 size
- * @64x64:                 64x64 size
- * @max-size:              Max supported size
- */
 static void run_size_tests(data_t *data, int w, int h)
 {
 	enum pipe pipe;
@@ -871,7 +925,7 @@ static void run_size_tests(data_t *data, int w, int h)
 				if (!valid_pipe_output_combo(data))
 					continue;
 
-				if (require_cursor_size(data, w, h)) {
+				if (!cursor_size_supported(data, w, h)) {
 					igt_info("Cursor size %dx%d not supported by driver\n", w, h);
 					continue;
 				}
@@ -1007,6 +1061,25 @@ static void run_tests_on_pipe(data_t *data)
 	igt_fixture
 		igt_remove_fb(data->drm_fd, &data->fb);
 
+	igt_describe("Check that sizes declared in SIZE_HINTS are accepted.");
+	igt_subtest_with_dynamic("cursor-size-hints") {
+		for_each_pipe_with_single_output(&data->display, pipe, data->output) {
+			if (execution_constraint(pipe))
+				continue;
+
+			data->pipe = pipe;
+
+			if (!valid_pipe_output_combo(data))
+				continue;
+
+			igt_dynamic_f("pipe-%s-%s",
+				      kmstest_pipe_name(pipe),
+				      data->output->name)
+				run_test(data, test_size_hints,
+					 data->cursor_max_w, data->cursor_max_h);
+		}
+	}
+
 	for (cursor_size = 32; cursor_size <= 512; cursor_size *= 2) {
 		int w = cursor_size;
 		int h = cursor_size;
@@ -1075,7 +1148,11 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 
 		igt_require_pipe_crc(data.drm_fd);
 
-		data.vblank_wait_count = is_msm_device(data.drm_fd) ? 2 : 1;
+		/* Wait for two more vblanks since cursor updates may not
+		 * synchronized to the same frame on AMD HW
+		 */
+		data.vblank_wait_count =
+			(is_msm_device(data.drm_fd) || is_amdgpu_device(data.drm_fd)) ? 2 : 1;
 	}
 
 	data.cursor_max_w = cursor_width;
