@@ -29,12 +29,25 @@
  * TEST: kms dither
  * Category: Display
  * Description: Test Dithering block status
+ * Driver requirement: i915, xe
+ * Functionality: colorspace, kms_gem_interop
+ * Mega feature: General Display Features
+ * Test category: functionality test
  */
+
 #include "igt.h"
 #include <fcntl.h>
 #include <termios.h>
 #include <unistd.h>
 
+/**
+ * SUBTEST: fb-8bpc-vs-panel-6bpc
+ * Description: Framebuffer BPC:8, Panel BPC:6, Expected Dither:Enable
+ *
+ * SUBTEST: fb-8bpc-vs-panel-8bpc
+ * Description: Framebuffer BPC:8, Panel BPC:8, Expected Dither:Disable
+ */
+
 IGT_TEST_DESCRIPTION("Test Dithering block status");
 
 /* Connector BPC */
@@ -63,13 +76,10 @@ typedef struct {
 /* Prepare test data. */
 static void prepare_test(data_t *data, igt_output_t *output, enum pipe p)
 {
-	igt_display_t *display = &data->display;
 	igt_pipe_t *pipe = &data->display.pipes[p];
 
 	igt_assert(pipe);
 
-	igt_display_reset(display);
-
 	data->primary =
 		igt_pipe_get_plane_type(pipe, DRM_PLANE_TYPE_PRIMARY);
 
@@ -85,7 +95,7 @@ static dither_status_t get_dither_state(data_t *data, enum pipe pipe)
 	dither_status_t status;
 
 	dir = igt_debugfs_dir(data->drm_fd);
-	igt_assert(dir >= 0);
+	igt_assert_lte(0, dir);
 
 	igt_require_intel(data->drm_fd);
 
@@ -186,46 +196,44 @@ static bool is_supported(igt_output_t *output)
 		igt_output_get_prop(output, IGT_CONNECTOR_MAX_BPC);
 }
 
-/**
- * SUBTEST: FB-8BPC-Vs-Panel-6BPC
- * Description: Framebuffer BPC:8, Panel BPC:6, Expected Dither:Enable
- * Driver requirement: i915, xe
- * Functionality: colorspace, kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: FB-8BPC-Vs-Panel-8BPC
- * Description: Framebuffer BPC:8, Panel BPC:8, Expected Dither:Disable
- * Driver requirement: i915, xe
- * Functionality: colorspace, kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
- */
 static void
 run_dither_test(data_t *data, int fb_bpc, int fb_format, int output_bpc)
 {
 	igt_output_t *output;
 	igt_display_t *display = &data->display;
 
+	igt_display_reset(display);
+
 	for_each_connected_output(display, output) {
 		enum pipe pipe;
 
-		if (!is_supported(output))
+		if (!is_supported(output)) {
+			igt_info("Output %s: Doesn't support \"max bpc\" property.\n",
+				 igt_output_name(output));
 			continue;
+		}
 
-		if (igt_get_output_max_bpc(data->drm_fd, output->name) < output_bpc)
+		if (igt_get_output_max_bpc(data->drm_fd, output->name) < output_bpc) {
+			igt_info("Output %s: Doesn't support %d-bpc.\n",
+				 igt_output_name(output), output_bpc);
 			continue;
+		}
 
 		for_each_pipe(display, pipe) {
-			if (igt_pipe_connector_valid(pipe, output)) {
-				igt_dynamic_f("pipe-%s-%s",
-					      kmstest_pipe_name(pipe), output->name)
-					test_dithering(data, pipe, output, fb_bpc,
-							fb_format, output_bpc);
+			igt_output_set_pipe(output, pipe);
 
-				/* One pipe is enough */
-				break;
+			if (!intel_pipe_output_combo_valid(display)) {
+				igt_output_set_pipe(output, PIPE_NONE);
+				continue;
 			}
+
+			igt_dynamic_f("pipe-%s-%s",
+					      kmstest_pipe_name(pipe), output->name)
+				test_dithering(data, pipe, output, fb_bpc,
+							   fb_format, output_bpc);
+
+			/* One pipe is enough */
+			break;
 		}
 	}
 }
@@ -257,7 +265,7 @@ igt_main
 			       (tests[i].fb_bpc > tests[i].output_bpc) ?
 							"Enable": "Disable");
 
-		igt_subtest_with_dynamic_f("FB-%dBPC-Vs-Panel-%dBPC",
+		igt_subtest_with_dynamic_f("fb-%dbpc-vs-panel-%dbpc",
 				tests[i].fb_bpc, tests[i].output_bpc)
 			run_dither_test(&data,
 					tests[i].fb_bpc,
