@@ -25,96 +25,87 @@
  * TEST: kms flip
  * Category: Display
  * Description: Tests for validating modeset, dpms and pageflips
+ * Driver requirement: i915, xe
+ * Functionality: vblank
+ * Mega feature: General Display Features
+ * Test category: functionality test
  */
 
 #include "config.h"
 
 #include "igt.h"
+#include "i915/intel_drrs.h"
 
 #include <cairo.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <math.h>
+#include <poll.h>
 #include <stdint.h>
 #include <unistd.h>
-#include <sys/poll.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
 
+#ifdef HAVE_LINUX_KD_H
+#include <linux/kd.h>
+#elif HAVE_SYS_KD_H
+#include <sys/kd.h>
+#endif
+#include <time.h>
+#include <pthread.h>
+
+#include "i915/gem_create.h"
+#include "igt_stats.h"
+#include "xe/xe_query.h"
+
 /**
- * SUBTEST: 2x-flip-vs-fences
- * Description: Test to validate pageflips along with avialable fences on a pair
- *              of connected displays
+ * SUBTEST: %s
+ * Description: %arg[1] test to validate pageflips with available fences
  * Driver requirement: i915
  * Functionality: gtt, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: 2x-flip-vs-fences-interruptible
- * Description: Interrupt test to validate pageflips along with available fences
+ * SUBTEST: 2x-%s
+ * Description: %arg[1] test to validate pageflips along with available fences
  *              on a pair of connected displays
- * Driver requirement: i915, xe
- * Functionality: gtt, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: flip-vs-fences
- * Description: Basic test to validate pageflips with avialable fences
  * Driver requirement: i915
  * Functionality: gtt, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: flip-vs-fences-interruptible
- * Description: Interrupt test to validate pageflips with available fences
- * Driver requirement: i915
- * Functionality: gtt, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
+ * arg[1]:
  *
- * SUBTEST: dpms-off-confusion
- * Description: Basic test to validate pageflips by disabling other connectors usng dpms
- * Driver requirement: i915, xe
+ * @flip-vs-fences:                 Basic
+ * @flip-vs-fences-interruptible:   Interrupt
+ */
+
+/**
+ * SUBTEST: dpms-off-%s
+ * Description: %arg[1] test to validate pageflips by disabling other connectors usng dpms
  * Functionality: dpms, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: dpms-off-confusion-interruptible
- * Description: Interrupt test to validate pageflips by disabling other connectors using dpms
- * Driver requirement: i915, xe
- * Functionality: dpms, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
+ * arg[1]:
  *
- * SUBTEST: bo-too-big
- * Description: Basic test to validate pageflips with large BO in size
- * Driver requirement: i915, xe
+ * @confusion:                      Basic
+ * @confusion-interruptible:        Interrupt
+ */
+
+/**
+ * SUBTEST: %s
+ * Description: %arg[1] test to validate pageflips with large BO in size
  * Functionality: kms_gem_interop, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: bo-too-big-interruptible
- * Description: Interrupt test to validate pageflips with large BO in size
- * Driver requirement: i915, xe
- * Functionality: kms_gem_interop, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
+ * arg[1]:
+ *
+ * @bo-too-big:                     Basic
+ * @bo-too-big-interruptible:       Interrupt
  */
 
 /**
  * SUBTEST: %s
  * Description: Basic test to validate %arg[1]
- * Driver requirement: i915, xe
  * Functionality: hang, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s
  * Description: Test to validate %arg[1] on a pair of connected displays
- * Driver requirement: i915, xe
  * Functionality: hang, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -125,17 +116,9 @@
 /**
  * SUBTEST: %s
  * Description: Basic test to validate %arg[1]
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s
  * Description: Test to validate %arg[1] on a pair of connected displays
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -156,63 +139,35 @@
  */
 
 /**
- * SUBTEST: flip-vs-suspend
- * Description: Basic test to validate pageflips with suspend cycle
- * Driver requirement: i915, xe
+ * SUBTEST: %s
+ * Description: %arg[1] test to validate pageflips with suspend cycle
  * Functionality: suspend, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
- * SUBTEST: 2x-flip-vs-suspend
- * Description: Basic test to validate pageflips with suspend cycle on a pair of
- *              connected displays
- * Driver requirement: i915, xe
- * Functionality: suspend, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: flip-vs-suspend-interruptible
- * Description: Interrupt test to validate pageflips with suspend cycle
- * Driver requirement: i915, xe
- * Functionality: suspend, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: 2x-flip-vs-suspend-interruptible
- * Description: Interrupt test to validate pageflips with suspend cycle on a pair
+ * SUBTEST: 2x-%s
+ * Description: %arg[1] test to validate pageflips with suspend cycle on a pair
  *              of connected displays
- * Driver requirement: i915, xe
  * Functionality: suspend, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
+ * arg[1]:
+ *
+ * @flip-vs-suspend:                   Basic
+ * @flip-vs-suspend-interruptible:     Interrupt
+ */
+
+/**
  * SUBTEST: %s
  * Description: Basic test to validate %arg[1]
- * Driver requirement: i915, xe
  * Functionality: dpms, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s
  * Description: Basic test to validate %arg[1] on a pair of connected displays
- * Driver requirement: i915, xe
  * Functionality: dpms, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: %s-interruptible
  * Description: Basic test to validate %arg[1]
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s-interruptible
  * Description: Basic test to validate %arg[1] on a pair of connected displays
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -227,17 +182,10 @@
  * SUBTEST: 2x-flip-vs-dpms
  * Description: Basic test to validate pageflip along with dpms on a pair of
  *              connected displays
- * Driver requirement: i915, xe
  * Functionality: dpms, vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s
  * Description: Basic test to validate %arg[1] on a pair of connected displays
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -249,18 +197,10 @@
 /**
  * SUBTEST: %s-interruptible
  * Description: Basic test for validating modeset, dpms and pageflips
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: 2x-%s-interruptible
  * Description: Test for validating modeset, dpms and pageflips with a pair of
  *              connected displays
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -279,17 +219,25 @@
  * @modeset-vs-vblank-race:       modeset and check for vblank
  */
 
-#ifdef HAVE_LINUX_KD_H
-#include <linux/kd.h>
-#elif HAVE_SYS_KD_H
-#include <sys/kd.h>
-#endif
-#include <time.h>
-#include <pthread.h>
-
-#include "i915/gem_create.h"
-#include "igt_stats.h"
-#include "xe/xe_query.h"
+/**
+ * SUBTEST: basic-plain-flip
+ * Description: Basic test for validating page flip
+ *
+ * SUBTEST: nonblocking-read
+ * Description: Tests that nonblocking reading fails correctly
+ *
+ * SUBTEST: basic-flip-vs-dpms
+ * Description: Basic test to valide pageflip with dpms
+ * Functionality: dpms, vblank
+ *
+ * SUBTEST: basic-flip-vs-%s
+ * Description: Basic test to valide pageflip with %arg[1]
+ *
+ * arg[1]:
+ *
+ * @modeset:      modeset
+ * @wf_vblank:    wait for vblank
+ */
 
 #define TEST_DPMS		(1 << 0)
 
@@ -327,8 +275,6 @@
 #define RUN_TEST		1
 #define RUN_PAIR		2
 
-#define MAX_HDISPLAY_PER_CRTC 5120
-
 #ifndef DRM_CAP_TIMESTAMP_MONOTONIC
 #define DRM_CAP_TIMESTAMP_MONOTONIC 6
 #endif
@@ -342,6 +288,7 @@ uint32_t devid;
 int test_time = 3;
 static bool monotonic_timestamp;
 static pthread_t vblank_wait_thread;
+static int max_dotclock;
 
 static drmModeConnector *last_connector;
 
@@ -476,8 +423,8 @@ static void emit_fence_stress(struct test_output *o)
 	igt_require(bops);
 
 	igt_assert(num_fences);
-	bo = calloc(sizeof(*bo), num_fences);
-	exec = calloc(sizeof(*exec), num_fences+1);
+	bo = calloc(num_fences, sizeof(*bo));
+	exec = calloc(num_fences+1, sizeof(*exec));
 	for (i = 0; i < num_fences - 1; i++) {
 		uint32_t tiling = I915_TILING_X;
 		bo[i] = intel_buf_create(bops, 1024, 1024, 32, 0, tiling,
@@ -911,6 +858,9 @@ static int set_mode(struct test_output *o, uint32_t fb, int x, int y)
 				     conn, count, mode);
 		if (ret)
 			return ret;
+
+		if (is_intel_device(drm_fd))
+			intel_drrs_disable(drm_fd, o->pipe);
 	}
 
 	return 0;
@@ -1778,21 +1728,39 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	igt_assert_eq(o->count, crtc_count);
 
 	/*
-	 * Handle BW limitations:
+	 * Handle BW limitations on intel hardware:
 	 *
-	 * if mode.hdisplay > 5120, then ignore
+	 * if force joiner (or) mode resolution > 5K (or) mode clock > max_dotclock, then ignore
 	 *  - last crtc in single/multi-connector config
 	 *  - consecutive crtcs in multi-connector config
 	 *
 	 * in multi-connector config ignore if
-	 *  - previous crtc mode.hdisplay > 5120 and
+	 *  - previous crtc (force joiner or mode resolution > 5K or mode clock > max_dotclock) and
 	 *  - current & previous crtcs are consecutive
 	 */
+	if (!is_intel_device(drm_fd))
+		goto test;
+
 	for (i = 0; i < crtc_count; i++) {
-		if (((o->kmode[i].hdisplay > MAX_HDISPLAY_PER_CRTC) &&
+		char conn_name[24], prev_conn_name[24];
+
+		snprintf(conn_name, sizeof(conn_name),
+			 "%s-%d",
+			 kmstest_connector_type_str(o->kconnector[i]->connector_type),
+			 o->kconnector[i]->connector_type_id);
+
+		if (i > 0)
+			snprintf(prev_conn_name, sizeof(prev_conn_name),
+				 "%s-%d",
+				 kmstest_connector_type_str(o->kconnector[i - 1]->connector_type),
+				 o->kconnector[i - 1]->connector_type_id);
+
+		if (((igt_check_force_joiner_status(drm_fd, conn_name) ||
+		      igt_bigjoiner_possible(drm_fd, &o->kmode[i], max_dotclock)) &&
 		     ((crtc_idxs[i] >= (total_crtcs - 1)) ||
 		      ((i < (crtc_count - 1)) && (abs(crtc_idxs[i + 1] - crtc_idxs[i]) <= 1)))) ||
-		    ((i > 0) && (o->kmode[i - 1].hdisplay > MAX_HDISPLAY_PER_CRTC) &&
+		    ((i > 0) && (igt_check_force_joiner_status(drm_fd, prev_conn_name) ||
+				 igt_bigjoiner_possible(drm_fd, &o->kmode[i - 1], max_dotclock)) &&
 		     (abs(crtc_idxs[i] - crtc_idxs[i - 1]) <= 1))) {
 
 			igt_debug("Combo: %s is not possible with selected mode(s).\n", test_name);
@@ -1800,38 +1768,11 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 		}
 	}
 
+test:
 	igt_dynamic_f("%s", test_name)
 		__run_test_on_crtc_set(o, crtc_idxs, crtc_count, duration_ms);
 }
 
-/**
- * SUBTEST: basic-flip-vs-dpms
- * Description: Basic test to valide pageflip with dpms
- * Driver requirement: i915, xe
- * Test category: functionality test
- * Functionality: dpms, vblank
- * Mega feature: General Display Features
- *
- * SUBTEST: basic-flip-vs-%s
- * Description: Basic test to valide pageflip with %arg[1]
- * Driver requirement: i915, xe
- * Test category: functionality test
- * Functionality: vblank
- * Mega feature: General Display Features
- *
- * arg[1]:
- *
- * @modeset:      modeset
- * @wf_vblank:    wait for vblank
- *
- * SUBTEST: basic-plain-flip
- * Description: Basic test for validating page flip
- * Driver requirement: i915, xe
- * Test category: functionality test
- * Functionality: vblank
- * Mega feature: General Display Features
- */
-
 static void run_test(int duration, int flags)
 {
 	struct test_output o;
@@ -1955,7 +1896,7 @@ static void run_pair(int duration, int flags)
 
 	/* If we have fewer than 2 connected outputs then we won't have any
 	 * configuration at all. So skip in that case. */
-	igt_require_f(modes, "At least two displays required\n");
+	igt_require_f(modes, "At least two displays with same modes are required\n");
 
 	if (duration) {
 		duration = duration * 1000 / modes;
@@ -2016,14 +1957,6 @@ static void kms_flip_exit_handler(int sig)
 		kmstest_set_connector_dpms(drm_fd, last_connector, DRM_MODE_DPMS_ON);
 }
 
-/**
- * SUBTEST: nonblocking-read
- * Description: Tests that nonblocking reading fails correctly
- * Driver requirement: i915, xe
- * Functionality: vblank
- * Mega feature: General Display Features
- * Test category: functionality test
- */
 static void test_nonblocking_read(int in)
 {
 	char buffer[1024];
@@ -2112,12 +2045,13 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 		{ 0, TEST_BO_TOOBIG | TEST_NO_2X_OUTPUT, "bo-too-big" },
 		{ 10, TEST_FLIP | TEST_SUSPEND, "flip-vs-suspend" },
 	};
+	igt_display_t display;
 	int i;
 
 	igt_fixture {
 		drm_fd = drm_open_driver_master(DRIVER_ANY);
 
-		igt_enable_connectors(drm_fd);
+		igt_display_require(&display, drm_fd);
 
 		kmstest_set_vt_graphics_mode();
 		igt_install_exit_handler(kms_flip_exit_handler);
@@ -2132,6 +2066,7 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 			for (i = 0; i < sizeof(tests) / sizeof(tests[0]); i++)
 				tests[i].flags &= ~(TEST_CHECK_TS | TEST_VBLANK_EXPIRED_SEQ);
 		}
+		max_dotclock = igt_get_max_dotclock(drm_fd);
 	}
 
 	igt_describe("Tests that nonblocking reading fails correctly");
@@ -2188,6 +2123,8 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 	}
 	igt_stop_signal_helper();
 
-	igt_fixture
+	igt_fixture {
+		igt_display_fini(&display);
 		drm_close_driver(drm_fd);
+	}
 }
