@@ -22,6 +22,12 @@
  *
  */
 
+/**
+ * TEST: kms flip event leak
+ * Category: Display
+ * Description: Test to validate flip event leak
+ */
+
 #include "igt.h"
 #include <errno.h>
 #include <stdbool.h>
@@ -32,10 +38,6 @@
 #include "xe/xe_query.h"
 
 /**
- * TEST: kms flip event leak
- * Category: Display
- * Description: Test to validate flip event leak
- *
  * SUBTEST: basic
  * Description: This test tries to provoke the kernel into leaking a pending
  *              page flip event when the fd is closed before the flip has
@@ -122,7 +124,7 @@ igt_main
 			igt_display_reset(&data.display);
 
 			igt_output_set_pipe(output, pipe);
-			if (!i915_pipe_output_combo_valid(&data.display))
+			if (!intel_pipe_output_combo_valid(&data.display))
 				continue;
 
 			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
