@@ -293,8 +293,11 @@ test_plane_position_with_output(data_t *data,
 	create_fb_for_mode(data, mode, &green, &rect, 1, &primary_fb);
 	igt_plane_set_fb(primary, &primary_fb);
 
-	if (!igt_plane_has_format_mod(sprite, DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR))
+	if (!igt_plane_has_format_mod(sprite, DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR)) {
+		igt_info("plane-%d does not supports XRGB format and Linear modifier\n",
+			 sprite->index);
 		return;
+	}
 
 	igt_create_color_fb(data->drm_fd,
 			    64, 64, /* width, height */
@@ -493,7 +496,7 @@ test_plane_panning(data_t *data, enum pipe pipe)
 		mode_found = true;
 		break;
 	}
-	igt_require(mode_found);
+	igt_require_f(mode_found, "All connector modes are skipped due to low memory\n");
 
 	if (data->flags & TEST_PANNING_TOP_LEFT)
 		test_grab_crc(data, output, pipe, &red, data->flags, &ref_crc);
@@ -1003,6 +1006,36 @@ static void check_allowed_plane_size_64x64(data_t *data, igt_plane_t *plane,
 	igt_remove_fb(data->drm_fd, &test_fb);
 }
 
+static bool skip_format_mod(data_t *data,
+			    uint32_t format, uint64_t modifier,
+			    struct igt_vec *tested_formats)
+{
+	/* igt doesn't know how to sw generate UBWC: */
+	if (is_msm_device(data->drm_fd) &&
+	    modifier == DRM_FORMAT_MOD_QCOM_COMPRESSED)
+		return true;
+
+	/* VEBOX just hangs with an actual 10bpc format */
+	if (igt_fb_is_gen12_mc_ccs_modifier(modifier) &&
+	    igt_reduce_format(format) == DRM_FORMAT_XRGB2101010)
+		return true;
+
+	/* test each format "class" only once in non-extended tests */
+	if (!data->extended && modifier != DRM_FORMAT_MOD_LINEAR) {
+		struct format_mod rf = {
+			.format = igt_reduce_format(format),
+			.modifier = modifier,
+		};
+
+		if (igt_vec_index(tested_formats, &rf) >= 0)
+			return true;
+
+		igt_vec_push(tested_formats, &rf);
+	}
+
+	return false;
+}
+
 static bool test_format_plane(data_t *data, enum pipe pipe,
 			      igt_output_t *output, igt_plane_t *plane, igt_fb_t *primary_fb)
 {
@@ -1076,33 +1109,18 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 			.modifier = plane->modifiers[i],
 		};
 
-		/* igt doesn't know how to sw generate UBWC: */
-		if (is_msm_device(data->drm_fd) &&
-		    f.modifier == DRM_FORMAT_MOD_QCOM_COMPRESSED)
-			continue;
-
 		if (f.format == ref.format &&
 		    f.modifier == ref.modifier)
 			continue;
 
-		/* test each format "class" only once in non-extended tests */
-		if (!data->extended && f.modifier != DRM_FORMAT_MOD_LINEAR) {
-			struct format_mod rf = {
-				.format = igt_reduce_format(f.format),
-				.modifier = f.modifier,
-			};
-
-			if (igt_vec_index(&tested_formats, &rf) >= 0) {
-				igt_debug("Skipping format " IGT_FORMAT_FMT " / modifier "
-					IGT_MODIFIER_FMT " on %s.%u\n",
-					IGT_FORMAT_ARGS(f.format),
-					IGT_MODIFIER_ARGS(f.modifier),
-					kmstest_pipe_name(pipe),
-					plane->index);
-				continue;
-			}
-
-			igt_vec_push(&tested_formats, &rf);
+		if (skip_format_mod(data, f.format, f.modifier, &tested_formats)) {
+			igt_debug("Skipping format " IGT_FORMAT_FMT " / modifier "
+				  IGT_MODIFIER_FMT " on %s.%u\n",
+				  IGT_FORMAT_ARGS(f.format),
+				  IGT_MODIFIER_ARGS(f.modifier),
+				  kmstest_pipe_name(pipe),
+				  plane->index);
+			continue;
 		}
 
 		if (f.format == DRM_FORMAT_C8) {
@@ -1248,7 +1266,7 @@ static void test_planar_settings(data_t *data)
 	 * If here is added non-intel tests below require will need to be
 	 * changed to if(..)
 	 */
-	igt_require(data->display.is_atomic);
+	igt_require_f(data->display.is_atomic, "Atomic mode-set not supported\n");
 	igt_require_intel(data->drm_fd);
 	devid = intel_get_drm_devid(data->drm_fd);
 	igt_require(intel_display_ver(devid) >= 9);
