@@ -24,18 +24,21 @@
  *     Swati Sharma <swati2.sharma@intel.com>
  */
 
-#include "igt.h"
 /**
  * TEST: kms scaling modes
  * Category: Display
  * Description: Test display scaling modes
- *
- * SUBTEST: scaling-mode-%s
- * Description: Tests %arg[1] display scaling mode
  * Driver requirement: i915, xe
- * Functionality: eDP, plane, scaling
+ * Functionality: edp, plane, scaling
  * Mega feature: General Display Features
  * Test category: functionality test
+ */
+
+#include "igt.h"
+
+/**
+ * SUBTEST: scaling-mode-%s
+ * Description: Tests %arg[1] display scaling mode
  *
  * arg[1]:
  *
@@ -118,10 +121,10 @@ static void test_scaling_mode(data_t *data, uint32_t flags)
 			continue;
 
 		igt_output_set_pipe(output, pipe);
-		if (!i915_pipe_output_combo_valid(display))
+		if (!intel_pipe_output_combo_valid(display))
 			continue;
 
-		igt_dynamic_f("%s-pipe-%s", output->name, kmstest_pipe_name(pipe))
+		igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
 			test_scaling_mode_on_output(display, pipe, output, flags);
 	}
 }
