@@ -45,6 +45,13 @@
  *  - DP commands (e.g. poweroff)
  * - verify outputs against VBT/physical connectors
  */
+
+/**
+ * TEST: testdisplay
+ * Category: Display
+ * Description: Tests basic display functionality.
+ */
+
 #include "config.h"
 
 #include "igt.h"
@@ -57,8 +64,8 @@
 #include <stdbool.h>
 #include <strings.h>
 #include <unistd.h>
+#include <poll.h>
 #include <termios.h>
-#include <sys/poll.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
@@ -70,10 +77,6 @@
 #include <signal.h>
 
 /**
- * TEST: testdisplay
- * Category: Display
- * Description: Tests basic display functionality.
- *
  * SUBTEST:
  * Description: This test is intended for testing of display functionality like
  *              modeset, clone modes, test patterns & pixel generators etc..
@@ -82,6 +85,7 @@
  * Mega feature: General Display Features
  * Test category: functionality test
  */
+
 IGT_TEST_DESCRIPTION("Tests basic display functionality.");
 
 enum {
