@@ -26,18 +26,18 @@
 #include "igt_debugfs.h"
 #include "igt_sysfs.h"
 
+#include <poll.h>
 #include <sys/mman.h>
-#include <sys/poll.h>
 #include <sys/stat.h>
 #include <dirent.h>
 /**
  * TEST: vgem slow
- * Category: Infrastructure
  * Description: Extended sanity check of Virtual GEM module (vGEM).
- * Feature: vgem
- * Functionality: mock device
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
  * Sub-category: DRM
+ * Functionality: mock device
+ * Feature: vgem
  * Test category: GEM_Legacy
  *
  * SUBTEST: nohang
